#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Security.Principal;
using System.Text;
using System.Workflow.Activities;

using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.DataAccess;
using EPiServer.Personalization;
using EPiServer.Security;
using EPiServer.WorkflowFoundation.Activities;
using EPiServer;
using EPiServer.WorkflowFoundation;


namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Contains some common functionality for approval workflows
    /// </summary>
    /// <remarks>This base classe is not abstract since workflow designer will fail
    /// to load workflows with abstract base classes.</remarks>
    public class ApprovalBase : StateMachineWorkflowActivity
    {
        /// <summary>
        /// Helper method that tries to translate a string to a users language.
        /// If language can not be determined the first enabled language branch will be used as fallback.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="languagekey">The key to the item in a language xml that should be resolved.</param>
        /// <returns>The translation for the given languageKey.</returns>
        public static string Translate(string userName, string languagekey)
        {
            //Get profile for user to get his/her language
            EPiServerProfile user = EPiServerProfile.Get(userName);
            string userLanguage = user == null ? null : user.Language;
            if (string.IsNullOrEmpty(userLanguage))
            {
                userLanguage = LanguageBranch.LoadFirstEnabledBranch().LanguageID;
            }

            return LanguageManager.Instance.Translate(languagekey, userLanguage);
        }

        /// <summary>
        /// Iterates over all tasks related to this instance and updates pagelink
        /// </summary>
        protected void UpdateTaskLinks(PageReference newLink)
        {
            IList<Task> tasks = Task.List(this.WorkflowInstanceId);
            foreach (Task task in tasks)
            {
                task.Subject = Translate(task.AssignedTo, "/workflows/approval/approvaltask/subject")
                   + " " + EPiServer.DataFactory.Instance.GetPage(newLink).PageName;
                task.StateBag["__pageLink"] = newLink;
                task.Save(false);
            }
        }

        /// <summary>
        /// Publishes the page.
        /// </summary>
        /// <param name="page">The page.</param>
        /// <remarks>The page will be published on behalf of the user who saved/created the version that is published.
        /// If StartPublish is set to a future date and there is already a published version of this language we dont publish the page
        /// since that would lead to that a previously published page will have none version published right now. in that case
        /// we set version to DelayedPublished meaning it will get published later when StartPublish date is reached.</remarks>
        protected void PublishPage(PageData page)
        {
            //impersonate to person who saved/created page
            string userName = string.IsNullOrEmpty(page.ChangedBy) ? page.CreatedBy : page.ChangedBy;
            IPrincipal oldPrincipal = PrincipalInfo.CurrentPrincipal;
            PrincipalInfo.CurrentPrincipal = PrincipalInfo.CreatePrincipal(userName);
            try
            {
                if (page != null)
                {
                    //See if PageStartPublish is a future time
                    if (page.StartPublish > DateTime.Now)
                    {
                        //see if there is published version of this language
                        LanguageSelector languageSelector = new LanguageSelector(page.LanguageBranch);    
                        PageData possiblyPublishedVersion = DataFactory.Instance.GetPage(page.PageLink.CreateReferenceToPublishedPage(), languageSelector);
                        if (possiblyPublishedVersion.CheckPublishedStatus(PagePublishedStatus.Published))
                        {
                            //Ok we found a published page on this language, then we dont publish but set to delayed publish
                            PropertyBoolean pageDelayedPublished = new PropertyBoolean(true);
                            pageDelayedPublished.IsModified = true;
                            page.Property["PageDelayedPublish"] = pageDelayedPublished;
                            EPiServer.DataFactory.Instance.Save(page, SaveAction.ForceCurrentVersion | SaveAction.CheckIn, EPiServer.Security.AccessLevel.NoAccess);
                            return;
                        }
                      
                    }
                    //Publish page
                    EPiServer.DataFactory.Instance.Save(page, EPiServer.DataAccess.SaveAction.Publish, EPiServer.Security.AccessLevel.NoAccess);
                }
            }
            catch
            {
                throw;
            }
            finally
            {
                PrincipalInfo.CurrentPrincipal = oldPrincipal;
            }
        }

        /// <summary>
        /// Sends an email to page owner about approved page.
        /// </summary>
        /// <param name="activity">The activity.</param>
        /// <param name="approvalEvent">The <see cref="CMS6Workflows.Workflows.ApprovalEventArgs"/> instance containing the event data.</param>
        /// <returns></returns>
        protected bool SendApprovedEmail(SendEmailActivity activity, ApprovalEventArgs approvalEvent)
        {
            //Get the approver
            EPiServerProfile fromUser = EPiServerProfile.Get(approvalEvent.Approver);
            if (fromUser == null || string.IsNullOrEmpty(fromUser.Email))
            {
                return false;
            }
            activity.From = fromUser.Email;

            //Get the page
            PageData page = EPiServer.DataFactory.Instance.GetPage(approvalEvent.PageLink);

            //Get the user should recive the email
            EPiServerProfile toUser = EPiServerProfile.Get(page.ChangedBy);
            if (toUser == null || string.IsNullOrEmpty(toUser.Email))
            {
                return false;
            }
            activity.To = toUser.Email;

            activity.Subject = string.Format(Translate(toUser.UserName, "/workflows/approval/approvedevent/pageapproved"), page.PageName);

            //Create the mail body
            activity.Body = string.Format(Translate(toUser.UserName, "/workflows/approval/approvedevent/approvedby"), fromUser.DisplayName);
            activity.Body += "\n";
            activity.Body += Translate(toUser.UserName, "/workflows/approval/approvedevent/comment") + ":\n";
            activity.Body += approvalEvent.Message;

            return true;
        }

        /// <summary>
        /// Handles the FailedSentEmail event of the sendEmailActivity control. It adds an entry to log.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void sendEmailActivity_FailedSentEmail(object sender, EventArgs e)
        {
            ErrorEventArgs ev = e as ErrorEventArgs;
            if (ev != null)
            {
                //if (AspNetWorkflowManager.log.IsErrorEnabled)
                //{
                //    AspNetWorkflowManager.log.Error(Resources.LogErrorsStrings.smtpSendFailed, ev.GetException());
                //}
            }
        }

        /// <summary>
        /// Gets the SMTP settings from configurationfile.
        /// </summary>
        /// <value>The SMTP from configurationfile.</value>
        public System.Net.Mail.SmtpClient SmtpFromConfigurationfile
        {
            get
            {
                return new System.Net.Mail.SmtpClient();
            }
        }

    }
}
