#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.Collections.Generic;
using System.Text;
using EPiServer.WorkflowFoundation.Interfaces;
using EPiServer.Personalization;
using EPiServer.WorkflowFoundation.Activities;
using EPiServer.Core;
using EPiServer.Security;
using System.Security.Principal;
using EPiServer.BaseLibrary;
using EPiServer.WorkflowFoundation;
using EPiServer;

namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Service registered with ExternalDataExchangeService. Used for communication beteen host instances for approval activities.
    /// </summary>
    public class ApprovalService : IApprovalCommunication
    {
        private static Dictionary<Guid, System.Threading.Timer> _callbacks = new Dictionary<Guid, System.Threading.Timer>(); 
        #region IApprovalCommunication Members
        /// <summary>
        /// Specifies the ApprovalEvent activity
        /// </summary>
        public event EventHandler<ApprovalEventArgs> ApprovalEvent;
        /// <summary>
        /// Specifies the SetApproversEvent activity
        /// </summary>
        public event EventHandler<SetApproversEventArgs> SetApproversEvent;
        #endregion

        public ApprovalService()
        {
            WorkflowSystem.WorkflowManager.PageRelatedEvent += new EventHandler<PageRelatedEventArgs>(WorkflowManager_PageRelatedEvent);
            WorkflowSystem.InstanceHandler.InstanceStarting += new EventHandler<StartInstanceEventArgs>(InstanceHandler_InstanceStarting);
            
            //Make sure custom types is registered with object store
            if (Context.Repository.SchemaForType(typeof(Workflows.ApproverItem)) == null)
            {
                Type type = typeof(Workflows.ApproverItem);
                EPiServer.Implementation.Serialization.TypeSchemaBuilder.RegisterSchemaAndType(type.ToString(), type);
            }
            if (Context.Repository.SchemaForType(typeof(List<Workflows.ApproverItem>)) == null)
            {
                Type type = typeof(List<Workflows.ApproverItem>);
                EPiServer.Implementation.Serialization.TypeSchemaBuilder.RegisterSchemaAndType(type.ToString(), type);
            }
        }

        //See if there is any approval workflow for the PageReference that is about to be published or deleted.
        //If so terminate them
        private void WorkflowManager_PageRelatedEvent(object sender, PageRelatedEventArgs e)
        {
            if (e.PageEvent == WorkflowPageEvent.PagePublished || e.PageEvent == WorkflowPageEvent.PageDeleted)
            {
                IList<WorkflowDefinitionInstance> instances = WorkflowSystem.InstanceHandler.GetInstances(e.PageEventArgs.PageLink);
                foreach (WorkflowDefinitionInstance instance in instances)
                {
                    if (instance.Type == typeof(CMS6Workflows.Workflows.SequentialApproval) ||
                        instance.Type == typeof(CMS6Workflows.Workflows.ParallelApproval))
                    {
                        if (e.PageEvent == WorkflowPageEvent.PageDeleted || (e.PageEvent == WorkflowPageEvent.PagePublished && e.PageEventArgs.PageLink == instance.PageLink))
                        {
                            EPiServer.WorkflowFoundation.Utility.DeleteTasksForInstance(instance.InstanceId);   
                            //Here we dont know if the page was published from workflow instance or from other reason.
                            //If it was published from workflow instance we dont want to call terminate but let instance
                            //complete normally. Therefore we set up callback timer so a "normal" approval instance gets
                            //chance to complete before we try to terminate instance.
                            lock (_callbacks)
                            {
                                System.Threading.Timer callBack = new System.Threading.Timer(TimerCallback, instance.InstanceId, 1000, System.Threading.Timeout.Infinite);
                                _callbacks.Add(instance.InstanceId, callBack);
                            }
                        }
                    }
                }
            }
        }

        private void TimerCallback(object state)
        {
            Guid instanceId = (Guid)state;
            lock (_callbacks)
            {
                if (_callbacks.ContainsKey(instanceId))
                {
                    _callbacks.Remove(instanceId);
                }
            }
            if (WorkflowSystem.InstanceHandler.ContainsInstance(instanceId))
            {
                WorkflowSystem.InstanceHandler.TerminateInstance(instanceId, "page published", AccessLevel.NoAccess);
            }
        }

        /// <summary>
        /// A new workflow instance is about to start. Check if it is a approval workflow and if
        /// so check so page version is not already published
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void InstanceHandler_InstanceStarting(object sender, StartInstanceEventArgs e)
        {
            //early return if this instance is already cancelled
            if (e.Cancel)
            {
                return;
            }
            if (e.WorkflowDefinition.Type == typeof(CMS6Workflows.Workflows.SequentialApproval) ||
                       e.WorkflowDefinition.Type == typeof(CMS6Workflows.Workflows.ParallelApproval))
            {
                object pageArgs;
                if (!e.NamedValueArgs.TryGetValue("PageArgs", out pageArgs) || !(pageArgs is WorkflowPageEventArgs))
                {
                    e.Cancel = true;
                    e.CancelReason = LanguageManager.Instance.Translate("/workflows/approval/startparameters/notvalidpage");
                    return;
                }
                else
                {
                    WorkflowPageEventArgs args = (WorkflowPageEventArgs)e.NamedValueArgs["PageArgs"];
                    try
                    {
                        PageData page = DataFactory.Instance.GetPage(args.PageLink);
                        if (page.CheckPublishedStatus(PagePublishedStatus.Published))
                        {
                            e.Cancel = true;
                            e.CancelReason = LanguageManager.Instance.Translate("/workflows/approval/startparameters/publishedversion");
                            return;
                        }
                        if (AlreadyRunningForThisLanguage(e, page))
                        {
                            return;
                        }
                    }
                    catch (PageNotFoundException)
                    {
                        e.Cancel = true;
                        e.CancelReason = LanguageManager.Instance.Translate("/workflows/approval/startparameters/notvalidpage") +
                            "id:" + args.PageLink.ToString();
                        return;
                    }
                   
                    CheckApprovers(e, args.PageLink);
                }
            }
        }

        /// <summary>
        /// Checks that there are not already running an approval workflow for this language of the page.
        /// </summary>
        /// <param name="e">The <see cref="EPiServer.WorkflowFoundation.StartInstanceEventArgs"/> instance containing the event data.</param>
        /// <param name="page">The page to start a approval workflow for</param>
        /// <returns></returns>
        private bool AlreadyRunningForThisLanguage(StartInstanceEventArgs e, PageData page)
        {
            IList<WorkflowDefinitionInstance> instances = WorkflowSystem.InstanceHandler.GetInstances(page.PageLink, EPiServer.Security.AccessLevel.NoAccess);
            
            foreach (WorkflowDefinitionInstance instance in instances)
            {
                if (instance.Type == typeof(SequentialApproval) || instance.Type == typeof(ParallelApproval))
                {
                    PageData existingApprovalVersion = DataFactory.Instance.GetPage(instance.PageLink);
                    if (String.Equals(page.LanguageBranch, existingApprovalVersion.LanguageBranch))
                    {
                        e.Cancel = true;
                        e.CancelReason = LanguageManager.Instance.Translate("/workflows/translate/cancelled");
                        return true;
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// Vill check that the given approvers has publish access. 
        /// And that if number of approvers is set that it is not larger than number of approvers 
        /// </summary>
        /// <param name="e"></param>
        /// <param name="pageLink"></param>
        private void CheckApprovers(StartInstanceEventArgs e, PageReference pageLink)
        {
            bool existGroups = false;
            List<ApproverItem> approvers;

            if (e.NamedValueArgs.ContainsKey("Approvers"))
            {
                approvers = (e.NamedValueArgs["Approvers"] as List<ApproverItem>) ?? new List<ApproverItem>();
            }
            else
            {
                approvers = new List<ApproverItem>();
            }

            StringBuilder usersWithoutPublishRights = new StringBuilder();

            foreach (ApproverItem approver in approvers)
            {
                existGroups = approver.IsRole ? true : existGroups;
                if (!HasReadAndPublishRights(approver.Approver, pageLink))
                {
                    e.Cancel = true;
                    usersWithoutPublishRights.Append(approver.Approver + ", ");
                }  
            }

            if (e.Cancel)
            {
                e.CancelReason = string.Format(System.Globalization.CultureInfo.InvariantCulture,
                   LanguageManager.Instance.Translate("/workflows/approval/startparameters/notpublishrights"),
                   usersWithoutPublishRights.ToString(),
                   pageLink.ToString());

                return;
            }

            //If there are no groups in approvers, check that minimum number of approvers is not larger than number of approvers
            if (e.NamedValueArgs.ContainsKey("NeededApprovments"))
            {
                if (!existGroups && (int)e.NamedValueArgs["NeededApprovments"] > approvers.Count)
                {
                    e.Cancel = true;
                    e.CancelReason = LanguageManager.Instance.Translate("/workflows/approval/startparameters/tolarge");
                }
            }
            else
            {
                //if no neededapprovments is set, set it to number of approvers.
                e.NamedValueArgs["NeededApprovments"] = approvers.Count;
            }
        }


        /// <summary>
        /// invokes the ApprovalEvent
        /// </summary>
        /// <param name="args"></param>
        public void InvokeApprovalEvent(ApprovalEventArgs args)
        {
            EventHandler<ApprovalEventArgs> approvalEvent = this.ApprovalEvent;
            if (approvalEvent != null)
            {
                approvalEvent(null, args);
            }
        }

        /// <summary>
        /// Invokes the SetApproversEvent
        /// </summary>
        /// <param name="args"></param>
        public void InvokeSetApproversEvent(SetApproversEventArgs args)
        {
            EventHandler<SetApproversEventArgs> approversEvent = this.SetApproversEvent;
            if (approversEvent != null)
            {
                approversEvent(null, args);
            }
        }
        

        /// <summary>
        /// Determines whether the specified has read and publish rights for the specified page
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="pageLink">The page link to check access for</param>
        /// <returns>
        /// 	<c>true</c> if has read and publish rights; otherwise, <c>false</c>.
        /// </returns>
        public bool HasReadAndPublishRights(string userName, PageReference pageLink)
        {
            PageData page = DataFactory.Instance.GetPage(pageLink);
            
            //see if approver is a group or user
            if (System.Web.Security.Roles.Enabled && System.Web.Security.Roles.RoleExists(userName))
            {
                bool read = false;
                bool publish = false;
                foreach (KeyValuePair<string, AccessControlEntry> aclEntry in page.ACL)
                {
                    if (aclEntry.Value.EntityType == SecurityEntityType.Role)
                    {
                        if (string.Compare(aclEntry.Key, userName, true) == 0 || string.Compare(aclEntry.Key, EveryoneRole.RoleName, true) == 0)
                        {
                            if (!read)
                            {
                                read = (aclEntry.Value.Access & AccessLevel.Read) == AccessLevel.Read;
                            }
                            if (!publish)
                            {
                                publish = (aclEntry.Value.Access & AccessLevel.Publish) == AccessLevel.Publish;
                            }
                        }
                    }
                    if (read && publish)
                    {
                        break;
                    }
                }
                return read && publish;
            }
            else
            {
                IPrincipal principal = PrincipalInfo.CreatePrincipal(userName);
                return page.GetSecurityDescriptor().HasAccess(principal, AccessLevel.Publish) && page.GetSecurityDescriptor().HasAccess(principal, AccessLevel.Read);
            }
        }
    }
}
