#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using EPiServer.WorkflowFoundation.UI;
using EPiServer.Core;
using EPiServer.WorkflowFoundation.Activities;
using EPiServer.WorkflowFoundation.Workflows;
using System.Collections.Generic;
using EPiServer.Personalization;
using System.Security.Principal;
using EPiServer.Security;
using System.Collections.ObjectModel;
using EPiServer.DataAbstraction;

namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Workflow for parallel approval. 
    /// This workflow will create tasks for all given approvers. When all or the specified minimum number of 
    /// approvers has approved the page the page gets published and the workflow completes. If some approver 
    /// disapproves a "not approved" task is created for person who saved/created page.
    /// </summary>
    /// <remarks>The workflow specifies the WorkflowPlugIn attribute to specify the usercontrol used to set start parameters.
    /// <para>
    /// By default a page published by this workflow will be published in the context
    /// of the user who saved the page. If another behaviour is wanted e.g. that the page should
    /// be published in the context of one of approvers the impersonation step in method OnPublishPage
    /// should be changed.
    /// </para></remarks>
    [WorkflowPlugIn(Area = EPiServer.PlugIn.PlugInArea.None, UrlFromUtil = "WorkflowsUI/WorkflowApprovalStart.ascx", LanguagePath = "/workflows/approval/parallel")]
    public sealed partial class ParallelApproval : ApprovalBase
    {
        // Registers dependecy properties so they can be databound

        /// <exclude/>
        public static DependencyProperty PageArgsProperty = DependencyProperty.Register("PageArgs", typeof(WorkflowPageEventArgs), typeof(ParallelApproval));
        /// <exclude/>
        public static DependencyProperty TaskDescriptionProperty = DependencyProperty.Register("TaskDescription", typeof(string), typeof(ParallelApproval));
        /// <exclude/>
        public static DependencyProperty ApproversProperty = DependencyProperty.Register("Approvers", typeof(List<ApproverItem>), typeof(ParallelApproval));
        /// <exclude/>
        public static DependencyProperty NeededApprovmentsProperty = DependencyProperty.Register("NeededApprovments", typeof(int), typeof(ParallelApproval));
        /// <exclude/>
        public static DependencyProperty ApprovedProperty = DependencyProperty.Register("Approved", typeof(bool), typeof(ParallelApproval));
        /// <exclude/>
        public static DependencyProperty ApprovalEventProperty = DependencyProperty.Register("ApprovalEvent", typeof(CMS6Workflows.Workflows.ApprovalEventArgs), typeof(ParallelApproval));

        //internal varible to keep track of how many tasks that been created
        private int _currentIndex;

        //keep tracks over how many that has currently approved page Dictionary<Approver, approved>
        private Dictionary<ApproverItem, bool> _approvments;


        /// <summary>
        /// Sets PageArgs as start parameters so we can get the page for automatically created workflows
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public WorkflowPageEventArgs PageArgs
        {
            get { return (WorkflowPageEventArgs)this.GetValue(ParallelApproval.PageArgsProperty); }
            set { this.SetValue(ParallelApproval.PageArgsProperty, value); }
        }

        /// <summary>
        /// Gets or sets the approvers for the workflow
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public List<ApproverItem> Approvers
        {
            get { return (List<ApproverItem>)this.GetValue(ParallelApproval.ApproversProperty); }
            set { this.SetValue(ParallelApproval.ApproversProperty, value); }
        }

        /// <summary>
        /// Gets or sets the task description.
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public string TaskDescription
        {
            get { return (string)this.GetValue(ParallelApproval.TaskDescriptionProperty); }
            set { this.SetValue(ParallelApproval.TaskDescriptionProperty, value); }
        }

        /// <summary>
        /// Gets or sets the minimum number of needed approvments.
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public int NeededApprovments
        {
            get { return (int)this.GetValue(ParallelApproval.NeededApprovmentsProperty); }
            set { this.SetValue(ParallelApproval.NeededApprovmentsProperty, value); }
        }

        /// <summary>
        /// Gets the page link for the page version this approval is concerning
        /// </summary>
        /// <value>The page link.</value>
        public PageReference PageLink
        {
            get { return PageArgs.PageLink; }
            set { PageArgs = new WorkflowPageEventArgs(PageArgs.EventType, value, PageArgs.TargetLink, PageArgs.CacheKey, PageArgs.InstanceId); }

        }

        /// <summary>
        /// Gets or sets the latest approval event.
        /// </summary>
        /// <value>The approval event.</value>
        public CMS6Workflows.Workflows.ApprovalEventArgs ApprovalEvent
        {
            get { return (CMS6Workflows.Workflows.ApprovalEventArgs)this.GetValue(ParallelApproval.ApprovalEventProperty); }
            set { this.SetValue(ParallelApproval.ApprovalEventProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="ParallelApproval"/> is approved.
        /// </summary>
        /// <value><c>true</c> if approved; otherwise, <c>false</c>.</value>
        public bool Approved
        {
            get { return (bool)this.GetValue(ParallelApproval.ApprovedProperty); }
            set { this.SetValue(ParallelApproval.ApprovedProperty, value); }
        }

        public ParallelApproval()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Code condition to determine if all tasks has been created
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NotAllTasksCreated(object sender, ConditionalEventArgs e)
        {
            e.Result = _currentIndex != Approvers.Count;
        }

        /// <summary>
        /// Called when [create approval task] activity is executed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCreateApprovalTask(object sender, EventArgs e)
        {
            CreateTask createTask = sender as CreateTask;

            createTask.TaskDescription = this.TaskDescription;
            createTask.AssignTo = Approvers[_currentIndex].Approver;
            createTask.TaskSubject = Translate(Approvers[_currentIndex].Approver, "/workflows/approval/approvaltask/subject")
                + " " + EPiServer.DataFactory.Instance.GetPage(PageLink).PageName;

            _currentIndex++;
        }

        /// <summary>
        /// Code condition to determine if more approvers are needed or if page can be published
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Workflow.Activities.ConditionalEventArgs"/> instance containing the event data.</param>
        private void IsMoreApprovers(object sender, ConditionalEventArgs e)
        {
            int numberOfApprovments = 0;
            foreach (KeyValuePair<ApproverItem, bool> approverPair in _approvments)
            {
                if (approverPair.Key.IsRequired && approverPair.Value == false)
                {
                    //If some required approver has not yet approved, we are not finished
                    e.Result = true;
                    return;
                }
                else if (approverPair.Value == true)
                {
                    numberOfApprovments++;
                }
            }

            //If number of approvments is less than minimum number more approvments are needed
            e.Result = numberOfApprovments < this.NeededApprovments;
        }

        /// <summary>
        /// Publishes the page.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void PublishPage(object sender, EventArgs e)
        {
            //We need a writable PageData to be able to save
            PageData page = EPiServer.DataFactory.Instance.GetPage(PageLink).CreateWritableClone();

            PublishPage(page);
        }

        /// <summary>
        /// Called when [create owner task] activity is executed, that is when some approver did not approve page
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCreateOwnerTask(object sender, EventArgs e)
        {
            CreateTask createtask = sender as CreateTask;

            PageData page = EPiServer.DataFactory.Instance.GetPage(PageLink);

            createtask.AssignTo = string.IsNullOrEmpty(page.ChangedBy) ? page.CreatedBy : page.ChangedBy;
            createtask.TaskDescription = this.ApprovalEvent.Message;
            createtask.TaskSubject = Translate(createtask.AssignTo, "/workflows/approval/approvedevent/notapproved") + " " + page.PageName;
        }

        /// <summary>
        /// Called when [set approvers] activity is executed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Workflow.Activities.ExternalDataEventArgs"/> instance containing the event data.</param>
        private void OnSetApprovers(object sender, ExternalDataEventArgs e)
        {
            SetApproversEventArgs args = e as SetApproversEventArgs;
            this.Approvers = args.Approvers;

            this.TaskDescription = args.TaskDescription;
            this.NeededApprovments = args.NeededApprovments;

            //check if page version has changed
            if (args.PageLink != PageArgs.PageLink)
            {
                //page version has changed
                PageLink = args.PageLink;
            }

            //reset internal variables
            _currentIndex = 0;
            InitializeApproversDictionary();
        }

        /// <summary>
        /// Initializes the internal dictionary that keeps track on which approvments that has been done
        /// </summary>
        private void InitializeApproversDictionary()
        {
            _approvments = new Dictionary<ApproverItem, bool>();
            foreach (ApproverItem item in Approvers)
            {
                _approvments.Add(item, false);
            }
        }

        /// <summary>
        /// called from code activity
        /// </summary>
        private void CreateApproverDictionary(object sender, EventArgs e)
        {
            InitializeApproversDictionary();
        }

        /// <summary>
        /// Called when [approval event] activity is executed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Workflow.Activities.ExternalDataEventArgs"/> instance containing the event data.</param>
        private void OnApprovalEvent(object sender, ExternalDataEventArgs e)
        {
            this.ApprovalEvent = e as ApprovalEventArgs;

            //check if page version has changed
            if (ApprovalEvent.PageLink != PageArgs.PageLink)
            {
                //page version has changed
                PageLink = ApprovalEvent.PageLink;
                UpdateTaskLinks(PageLink);
            }

            //If approved update dictionary and see if the conditions for publish is fulfilled.
            //if not approved do nothing since it will go to update state
            if (this.ApprovalEvent.Approved)
            {
                foreach (KeyValuePair<ApproverItem, bool> approverPair in _approvments)
                {
                    if (approverPair.Key.Approver == this.ApprovalEvent.TaskOwner &&
                        approverPair.Value == false)
                    {
                        _approvments[approverPair.Key] = true;
                        return;
                    }
                }

                //if not handled yet, then it must be second or later approver in a group, add person
                _approvments[new ApproverItem(this.ApprovalEvent.Approver, true, false)] = true;
            }
        }

        /// <summary>
        /// Called when [add history item] activity is executed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnAddHistoryItem(object sender, EventArgs e)
        {
            AddHistoryItem history = sender as AddHistoryItem;
            history.User = this.ApprovalEvent.Approver;
            history.ItemDescription = this.ApprovalEvent.Message;

            //set subject in not translated form, then they will be translated for 
            //the person who request the history item in his/her language
            history.ItemSubject = this.ApprovalEvent.Approved ?
                "/workflows/approval/history/approved" :
                "/workflows/approval/history/notapproved";
        }

        /// <summary>
        /// Called when [add approvers user data] activity is executed. Used so control loaded for task
        /// can be initialized with same approvers predefined as before.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnAddApproversUserData(object sender, EventArgs e)
        {
            AddUserData userData = sender as AddUserData;
            userData.Key = "Approvers";
            userData.Value = this.Approvers;
        }

        /// <summary>
        /// Called when [add numbers user data] activity is executed. Used so control loaded for task
        /// can be initialized with same number of minimum approvers predefined as before.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnAddNumbersUserData(object sender, EventArgs e)
        {
            AddUserData userData = sender as AddUserData;
            userData.Key = "NumbersOfApprovers";
            userData.Value = this.NeededApprovments;
        }

        /// <summary>
        /// Called when [add description user data] activity is executed. Used so control loaded for task
        /// can be initialized with same description predefined as before.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnAddDescriptionUserData(object sender, EventArgs e)
        {
            AddUserData userData = sender as AddUserData;
            userData.Key = "ApproversDescription";
            userData.Value = this.TaskDescription;
        }

        /// <summary>
        /// Code condition to see if there are any approvers specified
        /// </summary>
        private void HasApprovers(object sender, ConditionalEventArgs e)
        {
            e.Result = this.Approvers.Count > 0;
        }

        /// <summary>
        /// Code condition to see if latest approval was approved or not approved
        /// </summary>
        private void IsApproved(object sender, ConditionalEventArgs e)
        {
            e.Result = this.ApprovalEvent.Approved;
        }

        /// <summary>
        /// Validate start parameters
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void IsValidParameters(object sender, ConditionalEventArgs e)
        {
            e.Result = true;
            if (this.Approvers == null || this.Approvers.Count == 0)
            {
                e.Result = false;
            }
            if (this.PageArgs == null || PageReference.IsNullOrEmpty(this.PageArgs.PageLink))
            {
                e.Result = false;
            }
        }

        private void sendEmailActivity_SendingEmail(object sender, EventArgs e)
        {
            SendEmailActivity activity = sender as SendEmailActivity;
            CancelEventArgs cancelEvent = e as CancelEventArgs;
            if (activity != null && cancelEvent != null)
            {
                cancelEvent.Cancel = !SendApprovedEmail(activity, this.ApprovalEvent);
            }
        }

    }
}
