#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using EPiServer.WorkflowFoundation.UI;
using System.Collections.Generic;
using CMS6Workflows.Workflows;
using EPiServer.WorkflowFoundation.Activities;
using EPiServer.Core;
using EPiServer.Personalization;
using EPiServer.Security;
using System.Security.Principal;
using System.Collections.ObjectModel;
using EPiServer.DataAbstraction;
using System.IO;

namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Workflow for sequential approval. 
    /// The workflow will create tasks and waits for approval event foreach of the given approvers. 
    /// If an approver does not approve a "not approved task" will be created for the 
    /// person who saved page. When all approvers has approved the page will be published.
    /// </summary>
    /// <remarks>The workflow specifies the WorkflowPlugIn attribute to specify the usercontrol used to set start parameters.
    /// 
    /// By default a page published by this workflow will be published in the context
    /// of the user who saved the page. If another behaviour is wanted e.g. that the page should
    /// be published in the context of one of approvers the impersonation step in method OnPublishPage
    /// should be changed.</remarks>
    [WorkflowPlugIn(Area = EPiServer.PlugIn.PlugInArea.None, UrlFromUtil = "WorkflowsUI/WorkflowApprovalStart.ascx", LanguagePath = "/workflows/approval/sequential")]
    public sealed partial class SequentialApproval : ApprovalBase
    {
        // Registers dependecy properties so they can be databound

        /// <exclude/>
        public static DependencyProperty PageArgsProperty = DependencyProperty.Register("PageArgs", typeof(WorkflowPageEventArgs), typeof(SequentialApproval));
        /// <exclude/>
        public static DependencyProperty TaskDescriptionProperty = DependencyProperty.Register("TaskDescription", typeof(string), typeof(SequentialApproval));
        /// <exclude/>
        public static DependencyProperty ApproversProperty = DependencyProperty.Register("Approvers", typeof(List<ApproverItem>), typeof(SequentialApproval));
        /// <exclude/>
        public static DependencyProperty CurrentIndexProperty = DependencyProperty.Register("CurrentIndex", typeof(int), typeof(SequentialApproval));
        /// <exclude/>
        public static DependencyProperty ApprovedProperty = DependencyProperty.Register("Approved", typeof(bool), typeof(SequentialApproval));
        /// <exclude/>
        public static DependencyProperty ApprovalEventProperty = DependencyProperty.Register("ApprovalEvent", typeof(CMS6Workflows.Workflows.ApprovalEventArgs), typeof(SequentialApproval));

        /// <summary>
        /// Sets PageArgs as start parameters so we can get the page for automatically created workflows
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public WorkflowPageEventArgs PageArgs
        {
            get { return (WorkflowPageEventArgs)this.GetValue(SequentialApproval.PageArgsProperty); }
            set { this.SetValue(SequentialApproval.PageArgsProperty, value); }
        }

        /// <summary>
        /// Gets or sets the approvers.
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public List<ApproverItem> Approvers
        {
            get { return (List<ApproverItem>)this.GetValue(SequentialApproval.ApproversProperty); }
            set { this.SetValue(SequentialApproval.ApproversProperty, value); }
        }

        /// <summary>
        /// Gets or sets the task description.
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public string TaskDescription
        {
            get { return (string)this.GetValue(SequentialApproval.TaskDescriptionProperty); }
            set { this.SetValue(SequentialApproval.TaskDescriptionProperty, value); }
        }

        /// <summary>
        /// Gets the page link for the page version this approval is concerning
        /// </summary>
        public PageReference PageLink
        {
            get { return PageArgs.PageLink; }
            set { PageArgs = new WorkflowPageEventArgs(PageArgs.EventType, value, PageArgs.TargetLink, PageArgs.CacheKey, PageArgs.InstanceId); }
        }

        /// <summary>
        /// Gets or sets the index of the current approver.
        /// </summary>
        public int CurrentIndex
        {
            get { return (int)this.GetValue(SequentialApproval.CurrentIndexProperty); }
            set { this.SetValue(SequentialApproval.CurrentIndexProperty, value); }
        }

        /// <summary>
        /// Gets or sets the latest approval event.
        /// </summary>
        /// <value>The approval event.</value>
        public CMS6Workflows.Workflows.ApprovalEventArgs ApprovalEvent
        {
            get { return (CMS6Workflows.Workflows.ApprovalEventArgs)this.GetValue(SequentialApproval.ApprovalEventProperty); }
            set { this.SetValue(SequentialApproval.ApprovalEventProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="SequentialApproval"/> is approved.
        /// </summary>
        /// <value><c>true</c> if approved; otherwise, <c>false</c>.</value>
        public bool Approved
        {
            get { return (bool)this.GetValue(SequentialApproval.ApprovedProperty); }
            set { this.SetValue(SequentialApproval.ApprovedProperty, value); }
        }

        public SequentialApproval()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called when [create approval task] activity is executed.
        /// Creates an approval task for the current approver.
        /// </summary>
        /// <param name="sender">The sender</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCreateApprovalTask(object sender, EventArgs e)
        {
            CreateTask createTask = sender as CreateTask;

            createTask.AssignTo = Approvers[CurrentIndex].Approver;
            createTask.TaskDescription = this.TaskDescription;
            createTask.TaskSubject = Translate(Approvers[CurrentIndex].Approver, "/workflows/approval/approvaltask/subject")
                + " " + EPiServer.DataFactory.Instance.GetPage(PageLink).PageName;

        }

        /// <summary>
        /// Called when [approved event] activity is executed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Workflow.Activities.ExternalDataEventArgs"/> instance containing the event data.</param>
        private void OnApprovedEvent(object sender, ExternalDataEventArgs e)
        {
            this.ApprovalEvent = e as CMS6Workflows.Workflows.ApprovalEventArgs;
            //check if page version has changed
            if (ApprovalEvent.PageLink != PageArgs.PageLink)
            {
                //page version has changed
                PageLink = ApprovalEvent.PageLink;
                UpdateTaskLinks(PageLink);
            }
        }

        /// <summary>
        /// Called when [publish page] code activity is executed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnPublishPage(object sender, EventArgs e)
        {
            if (this.Approved)
            {
                //We need a writable PageData to be able to save
                PageData page = EPiServer.DataFactory.Instance.GetPage(PageLink).CreateWritableClone();

                PublishPage(page);
            }
        }


        /// <summary>
        /// Called when [create owner task] activity is executed.
        /// Creates an "not approved" task for the person who saved page.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnCreateOwnerTask(object sender, EventArgs e)
        {
            CreateTask createtask = sender as CreateTask;

            PageData page = EPiServer.DataFactory.Instance.GetPage(PageLink);

            createtask.AssignTo = string.IsNullOrEmpty(page.ChangedBy) ? page.CreatedBy : page.ChangedBy;
            createtask.TaskDescription = this.ApprovalEvent.Message;

            createtask.TaskSubject = Translate(createtask.AssignTo, "/workflows/approval/approvedevent/notapproved") + " " + page.PageName;
        }

        /// <summary>
        /// Called when [set approver] activity is executed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Workflow.Activities.ExternalDataEventArgs"/> instance containing the event data.</param>
        private void OnSetApprover(object sender, ExternalDataEventArgs e)
        {
            CMS6Workflows.Workflows.SetApproversEventArgs args = e as CMS6Workflows.Workflows.SetApproversEventArgs;
            this.Approvers = args.Approvers;
            this.TaskDescription = args.TaskDescription;

            //check if page version has changed
            if (args.PageLink != PageArgs.PageLink)
            {
                //page version has changed
                PageLink = args.PageLink;
            }

            CurrentIndex = 0;
        }

        /// <summary>
        /// Code condition to determine if more approvers are needed or if page can be published
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Workflow.Activities.ConditionalEventArgs"/> instance containing the event data.</param>
        private void IsMoreApprovers(object sender, ConditionalEventArgs e)
        {
            CurrentIndex++;
            if (Approvers.Count == CurrentIndex)
            {
                this.Approved = true;
                e.Result = false;
            }
            else
            {
                e.Result = true;
            }
        }

        /// <summary>
        /// Called when [add history item] activity is executed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnAddHistoryItem(object sender, EventArgs e)
        {
            AddHistoryItem history = sender as AddHistoryItem;
            history.User = this.ApprovalEvent.Approver;
            history.ItemDescription = this.ApprovalEvent.Message;

            //set strings in not translated form, then they will be translated for 
            //the person who request the history item in his/her language
            history.ItemSubject = this.ApprovalEvent.Approved ?
                "/workflows/approval/history/approved" :
                "/workflows/approval/history/notapproved";
        }

        /// <summary>
        /// Called when [add user data] activity is executed. 
        /// Used so control loaded when owner clicks on taks can be intialized with same value as previous
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <remarks>Used so usercontrol specified with task can be initialized with same approvers as before</remarks>
        private void OnAddUserData(object sender, EventArgs e)
        {
            AddUserData userData = sender as AddUserData;
            userData.Key = "Approvers";
            userData.Value = this.Approvers;
        }

        /// <summary>
        /// Called when [add description user data] activity is executed. Used so control loaded for task
        /// can be initialized with same description predefined as before.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnAddDescriptionUserData(object sender, EventArgs e)
        {
            AddUserData userData = sender as AddUserData;
            userData.Key = "ApproversDescription";
            userData.Value = this.TaskDescription;
        }

        /// <summary>
        /// code condition to see if there is approvers specified
        /// </summary>
        private void HasApprovers(object sender, ConditionalEventArgs e)
        {
            e.Result = this.Approvers.Count > 0;
        }

        /// <summary>
        /// code condition to see if last approvement was approved or not approved
        /// </summary>
        private void IsApproved(object sender, ConditionalEventArgs e)
        {
            e.Result = this.ApprovalEvent.Approved;
        }

        /// <summary>
        /// Validates that the start parameters are valid.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Workflow.Activities.ConditionalEventArgs"/> instance containing the event data.</param>
        private void IsValidParameters(object sender, ConditionalEventArgs e)
        {
            e.Result = true;
            if (this.Approvers == null || this.Approvers.Count == 0)
            {
                e.Result = false;
            }
            if (this.PageArgs == null || PageReference.IsNullOrEmpty(this.PageArgs.PageLink))
            {
                e.Result = false;
            }
        }

        private void sendEmailActivity_SendingEmail(object sender, EventArgs e)
        {
            SendEmailActivity activity = sender as SendEmailActivity;
            CancelEventArgs cancelEvent = e as CancelEventArgs;
            if (activity != null && cancelEvent != null)
            {
                cancelEvent.Cancel = !SendApprovedEmail(activity, this.ApprovalEvent);
            }
        }
    }
}
