#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using System.Collections.Generic;
using EPiServer.WorkflowFoundation.Activities;
using EPiServer.Core;
using EPiServer.Personalization;
using EPiServer.WorkflowFoundation.UI;
using System.Collections.ObjectModel;
using EPiServer.DataAbstraction;
using EPiServer;
using EPiServer.WorkflowFoundation;

namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Workflow for request for feedback. 
    /// This workflow will create a task where feedback is requested for all specified users. It will also create a task for
    /// workflow owner where he/she can see the recieved feedback. The owner can complete workflow at will or the workflow
    /// completes after specified time.
    /// </summary>
    /// <remarks>The workflow specifies the WorkflowPlugIn attribute to specify the usercontrol used to set start parameters.</remarks>
    [WorkflowPlugIn(Area = EPiServer.PlugIn.PlugInArea.None, UrlFromUtil = "WorkflowsUI/WorkflowFeedbackStart.ascx", LanguagePath = "/workflows/feedbackrequest/plugin")]
	public sealed partial class RequestForFeedback: SequentialWorkflowActivity
	{
        // Registers dependecy properties so they can be databound

        /// <exclude/>
        public static DependencyProperty PageArgsProperty = DependencyProperty.Register("PageArgs", typeof(WorkflowPageEventArgs), typeof(RequestForFeedback));
        /// <exclude/>
        public static DependencyProperty UsersProperty = DependencyProperty.Register("Users", typeof(List<UserItem>), typeof(RequestForFeedback));
        /// <exclude/>
        public static DependencyProperty OwnerProperty = DependencyProperty.Register("Owner", typeof(string), typeof(RequestForFeedback));
        /// <exclude/>
        public static DependencyProperty RequestDescriptionProperty = DependencyProperty.Register("RequestDescription", typeof(string), typeof(RequestForFeedback));
        /// <exclude/>
        public static DependencyProperty FileSystemArgsProperty = DependencyProperty.Register("FileSystemArgs", typeof(WorkflowFileSystemEventArgs), typeof(RequestForFeedback));


        private TimeSpan _timeToLive = new TimeSpan(120, 0, 0); //Default is 120 hours (5 days)
        
        //track which tasks that have been created
        private int _currentIndex;

        public RequestForFeedback()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Sets PageArgs as start parameters so we can get the page for automatically created workflows
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public WorkflowPageEventArgs PageArgs
        {
            get { return (WorkflowPageEventArgs)this.GetValue(RequestForFeedback.PageArgsProperty); }
            set { this.SetValue(RequestForFeedback.PageArgsProperty, value); }
        }

        public WorkflowFileSystemEventArgs FileSystemArgs
        {
            get { return (WorkflowFileSystemEventArgs)this.GetValue(RequestForFeedback.FileSystemArgsProperty); }
            set { this.SetValue(RequestForFeedback.FileSystemArgsProperty, value); }
        }

        /// <summary>
        /// Gets or sets the users the feedback request is assigned to
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public List<UserItem> Users
        {
            get 
            {
                if (this.GetValue(RequestForFeedback.UsersProperty) != null)
                {
                    return (List<UserItem>)this.GetValue(RequestForFeedback.UsersProperty);
                }
                else
                {
                    return new List<UserItem>();
                }
            }
            set { this.SetValue(RequestForFeedback.UsersProperty, value); }
        }

        /// <summary>
        /// Gets or sets owner of the workflow, that is the user that wants feedback
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public string Owner
        {
            get { return (string)this.GetValue(RequestForFeedback.OwnerProperty); }
            set { this.SetValue(RequestForFeedback.OwnerProperty, value); }
        }

        /// <summary>
        /// Gets or sets the description for the feedback request
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public string RequestDescription
        {
            get { return (string)this.GetValue(RequestForFeedback.RequestDescriptionProperty); }
            set { this.SetValue(RequestForFeedback.RequestDescriptionProperty, value); }
        }

        /// <summary>
        /// Gets or sets the time before the workflow completes
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public TimeSpan TimeToLive
        {
            get { return _timeToLive; }
            set { _timeToLive = value; }
        }

        /// <summary>
        /// Gets the page the request is associated with (if any)
        /// </summary>
        public PageReference PageLink
        {
            get
            {
                return PageArgs != null ? PageArgs.PageLink : PageReference.EmptyReference;
            }
        }
             
        /// <summary>
        /// Validate start parameters
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void IsValidParameters(object sender, ConditionalEventArgs e)
        {
            e.Result = true;
            if (Users.Count == 0 || string.IsNullOrEmpty(Owner))
            {
                e.Result = false;
            }
        }

        /// <summary>
        /// Checks if all tasks are created
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MoreUsers(object sender, ConditionalEventArgs e)
        {
            e.Result = _currentIndex != Users.Count;
        }

        /// <summary>
        /// Sets the time for when feedback can be posted
        /// </summary>
        private void setTimeToLive(object sender, EventArgs e)
        {
            DelayActivity delay = sender as DelayActivity;
            delay.TimeoutDuration = TimeToLive;
        }

        /// <summary>
        /// Creates a feedback request in form of a task
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnCreateFeedbackRequests(object sender, EventArgs e)
        {
            CreateTask createTask = sender as CreateTask;
            createTask.TaskDescription = this.RequestDescription;
            createTask.AssignTo = Users[_currentIndex].UserName;
            createTask.AssociatedActivity = this.feedbackUserActivity1;
            createTask.DueDate = DateTime.Now.Add(TimeToLive);

            string subject = string.Empty;
            if (createTask.PageLink != null)
            {
                subject = " " + DataFactory.Instance.GetPage(createTask.PageLink).PageName;
            }
            else if(FileSystemArgs != null)
            {
                subject = " " + FileSystemArgs.NewVirtualPath;
            }
            createTask.TaskSubject = ApprovalBase.Translate(createTask.AssignTo, "/workflows/feedbackrequest/subject") + subject;
            
            _currentIndex++;
        }

        /// <summary>
        /// Creates a task for owner where he/she can see the feedback
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnCreateOwnerFeedback(object sender, EventArgs e)
        {
            CreateTask createTask = sender as CreateTask;
            createTask.AssignTo = Owner;
            createTask.TaskDescription = this.RequestDescription;
            createTask.StateBag.Add("Owner", true);
            createTask.TaskSubject = ApprovalBase.Translate(createTask.AssignTo, "/workflows/feedbackrequest/ownersubject");
        }

        /// <summary>
        /// Clear all tasks except for owner
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RemoveTasks(object sender, EventArgs e)
        {
            IList<Task> tasks = WorkflowSystem.InstanceHandler.GetTasks(this.WorkflowInstanceId);
            foreach (Task task in tasks)
            {
                if (task.StateBag.Contains("Owner") && ((bool)task.StateBag["Owner"]) == true)
                {
                    continue;
                }
                Task.Delete(task.TaskID);
            }
        }
	}

}
