#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Globalization;
using System.Text;
using System.Web.Security;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.Runtime;

using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.Personalization;
using EPiServer.WorkflowFoundation.Activities;
using EPiServer.WorkflowFoundation.UI;
using EPiServer;
using EPiServer.WorkflowFoundation;

namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Workflow for ready for translation of page. 
    /// This workflow will create a task with a translation request for each specified language. If a main translator is specified
    /// tasks will be created for that user for each not specified (but enabled on site) language.
    /// After the final time specified the workflow will notify owner of which pages that was translated before it completes.
    /// </summary>
    /// <remarks>The workflow specifies the WorkflowPlugIn attribute to specify the usercontrol used to set start parameters.</remarks>
    [WorkflowPlugIn(Area = EPiServer.PlugIn.PlugInArea.None, UrlFromUtil = "WorkflowsUI/WorkflowTranslationStart.ascx", LanguagePath = "/workflows/translate/plugin")]
	public sealed partial class ReadyForTranslation: SequentialWorkflowActivity
	{
        // Registers dependecy properties so they can be databound

        /// <exclude/>
        public static DependencyProperty PageArgsProperty = DependencyProperty.Register("PageArgs", typeof(WorkflowPageEventArgs), typeof(ReadyForTranslation));
        /// <exclude/>
        public static DependencyProperty TranslatorsProperty = DependencyProperty.Register("Translators", typeof(List<TranslationItem>), typeof(ReadyForTranslation));
        /// <exclude/>
        public static DependencyProperty MainTranslatorProperty = DependencyProperty.Register("MainTranslator", typeof(string), typeof(ReadyForTranslation));
        /// <exclude/>
        public static DependencyProperty OwnerProperty = DependencyProperty.Register("Owner", typeof(string), typeof(ReadyForTranslation));


        private DateTime _startTime = DateTime.Now;
        private TimeSpan _initialDelay = new TimeSpan(0, 10, 0); //Default is 10 minutes
        private TimeSpan _reminder = new TimeSpan(48, 0, 0); //Default is 48 hours
        private TimeSpan _final = new TimeSpan(120, 0, 0); //Default is 120 hours/5 days
        
        //keeps track over which tasks that have been created
        private int _currentIndex;

        //indicates if page needs to be published to fulfil task
        private bool _publishNeeded;

        //The translators that tasks have been sent to
        private List<TranslationItem> _determinedTranslators;

        //keeps track over which languages that have been translated
        private Dictionary<TranslationItem, bool> _translateTasks;

        /// <summary>
        /// Sets PageArgs as start parameters so we can get the page for automatically created workflows
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public WorkflowPageEventArgs PageArgs
        {
            get { return (WorkflowPageEventArgs)this.GetValue(ReadyForTranslation.PageArgsProperty); }
            set { this.SetValue(ReadyForTranslation.PageArgsProperty, value); }
        }

        /// <summary>
        /// Indicates if page needs to be published to fullfil task
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public bool PublishNeeded
        {
            get { return _publishNeeded; }
            set { _publishNeeded = value; }
        }

        /// <summary>
        /// The specified translators
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public List<TranslationItem> Translators
        {
            get 
            {
                if (this.GetValue(ReadyForTranslation.TranslatorsProperty) != null)
                {
                    return (List<TranslationItem>)this.GetValue(ReadyForTranslation.TranslatorsProperty);
                }
                else
                {
                    return new List<TranslationItem>(); 
                }
            }
            set { this.SetValue(ReadyForTranslation.TranslatorsProperty, value); }
        }

        /// <summary>
        /// The fallback translator for site enabled languages that is not explictly assigned
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public string MainTranslator
        {
            get { return (string)this.GetValue(ReadyForTranslation.MainTranslatorProperty); }
            set { this.SetValue(ReadyForTranslation.MainTranslatorProperty, value); }
        }

        /// <summary>
        /// The owner of the translation process
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public string Owner
        {
            get { return (string)this.GetValue(ReadyForTranslation.OwnerProperty); }
            set { this.SetValue(ReadyForTranslation.OwnerProperty, value); }
        }

        /// <summary>
        /// The initial delay before translation request tasks is created
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public TimeSpan InitialDelay
        {
            get { return _initialDelay; }
            set { _initialDelay = value; }
        }

        /// <summary>
        /// When a reminder should be sent fore none fulfilled translation tasks
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public TimeSpan Reminder
        {
            get { return _reminder; }
            set { _reminder = value; }
        }

        /// <summary>
        /// When the workflow should complete
        /// </summary>
        /// <remarks>set as startparameter when instance is created</remarks>
        public TimeSpan Final
        {
            get { return _final; }
            set { _final = value; }
        }

        /// <summary>
        /// The name of the page that is translated
        /// </summary>
        public string PageName
        {
            get
            {
                return EPiServer.DataFactory.Instance.GetPage(PageArgs.PageLink).PageName;
            }
        }

        /// <summary>
        /// The language of the version of the page that initially started the workflow
        /// </summary>
        public string InitialLanguage
        {
            get
            {
                return EPiServer.DataFactory.Instance.GetPage(PageArgs.PageLink).LanguageID;
            }
        }


		public ReadyForTranslation()
		{
			InitializeComponent();
        }

        /// <summary>
        /// Validates startparameters
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ValidateStartCondition(object sender, ConditionalEventArgs e)
        {
            e.Result = true;
            if (PageArgs == null || PageReference.IsNullOrEmpty(PageArgs.PageLink))
            {
                e.Result = false;
            }

            if (Translators.Count == 0 && string.IsNullOrEmpty(MainTranslator))
            {
                e.Result = false;
            }
        }

        /// <summary>
        /// Sets the intial delay before request for translation is sent out
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetInitialDelay(object sender, EventArgs e)
        {
            DelayActivity delay = sender as DelayActivity;
            delay.TimeoutDuration = this.InitialDelay;
        }

        /// <summary>
        /// Code condition to decide if there are more translators
        /// </summary>
        private void IsMoreTranslators(object sender, ConditionalEventArgs e)
        {
            e.Result = _currentIndex != _determinedTranslators.Count;
        }

        /// <summary>
        /// Goes through all enabled languages for the site and the specified translators
        /// and sets upp translators
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DetermineTranslators(object sender, EventArgs e)
        {
            _determinedTranslators = new List<TranslationItem>();
            _translateTasks = new Dictionary<TranslationItem, bool>();

            //Determine which languages are registered on site
            LanguageBranchCollection languages = LanguageBranch.ListAll();
            foreach (LanguageBranch language in languages)
            {
                if (language.Enabled && language.LanguageID != InitialLanguage)
                {
                    bool found = false;
                    foreach (TranslationItem item in Translators)
                    {
                        if (item.LanguageId == language.LanguageID)
                        {
                            _determinedTranslators.Add(item);
                            found = true;
                            break;
                        }
                    }
                    if (!found && !string.IsNullOrEmpty(MainTranslator))
                    {
                        _determinedTranslators.Add(new TranslationItem(MainTranslator, language.Name, language.LanguageID));
                    }
                }
            }
        }

        /// <summary>
        /// when a translation request is to be made
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnCreateTranslationTask(object sender, EventArgs e)
        {
            TranslationItem item = _determinedTranslators[_currentIndex];
            _currentIndex++;

            CreateTask task = sender as CreateTask;
            string translator = item.Translator;
            task.AssignTo = translator;

            string pageName = DataFactory.Instance.GetPage(task.PageLink).PageName;

            task.TaskSubject = ApprovalBase.Translate(translator, "/workflows/translate/initialsubject") + " " + pageName;
            task.TaskDescription = string.Format(System.Globalization.CultureInfo.InvariantCulture,
                "{0} {1} {2} {3}",
                ApprovalBase.Translate(translator, "/workflows/translate/initialdescription"),
                PageName,
                ApprovalBase.Translate(translator, "/workflows/translate/to"),
                item.LanguageName);
           
            task.StateBag.Add("Translator", item);
            //Since reminder will be sent from task after 0.75*DueDate we correct for that
            double hours = _reminder.TotalHours * 4 / 3;
            task.DueDate = DateTime.Now.AddHours(hours);

            _translateTasks.Add(item, false);
        }

        /// <summary>
        /// Determines if workflow instance is considered comleted
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NotCompleted(object sender, ConditionalEventArgs e)
        {
            e.Result = true;
            //Has final time elapsed
            TimeSpan elapsedTime = DateTime.Now - _startTime;
            if (elapsedTime > _final)
            {
                e.Result = false;
            }
            else
            {
                //Check if page has been translated to all pages
                bool missesTranslation = false;
                foreach (KeyValuePair<TranslationItem, bool> languageItem in _translateTasks)
                {
                    if (languageItem.Value == false)
                    {
                        missesTranslation = true;
                        break;
                    }
                }
                e.Result = missesTranslation;
            }
        }

        /// <summary>
        /// Set remaining timeout
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetTimeout(object sender, EventArgs e)
        {
            DelayActivity delay = sender as DelayActivity;
            TimeSpan elapsedTime = DateTime.Now - _startTime;

            if (elapsedTime < _final)
            {
                delay.TimeoutDuration = _final - elapsedTime;
            }
            else
            {
                //final time has elapsed (should not occur) set delay to 1 sec
                delay.TimeoutDuration = new TimeSpan(0, 0, 1);
            }
        }

        /// <summary>
        /// Determines if a user is assigned translators. That is it checks if user os same
        /// or if user belongs to assigned group
        /// </summary>
        /// <param name="user"></param>
        /// <param name="assignedTranslator"></param>
        /// <returns></returns>
        private static bool IsAssignedTranslator(string user, string assignedTranslator)
        {
            if (string.Compare(user, assignedTranslator, true, System.Globalization.CultureInfo.InvariantCulture) == 0)
            {
                return true;
            }
            if (System.Web.Security.Roles.Enabled)
            {
                string[] roles = System.Web.Security.Roles.GetRolesForUser(user);
                foreach (string role in roles)
                {
                    if (string.Compare(role, assignedTranslator, true, System.Globalization.CultureInfo.InvariantCulture) == 0)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// Removes a task
        /// </summary>
        /// <param name="translationItem"></param>
        private void RemoveTask(TranslationItem translationItem)
        {
            IList<Task> tasks = Task.List(this.WorkflowInstanceId);
            foreach (Task task in tasks)
            {
                if (task.StateBag.Contains("Translator"))
                {
                    TranslationItem item = (TranslationItem)task.StateBag["Translator"];
                    if (item.LanguageId == translationItem.LanguageId)
                    {
                        Task.Delete(task.TaskID);
                        return;
                    }
                }
            }
        }

        /// <summary>
        /// Creates feedback to workflow owner
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnCreateOwnerFeedback(object sender, EventArgs e)
        {
            CreateTask task = sender as CreateTask;

            //Get languages that was translated and not translated
            StringBuilder translatedLanguagesBuilder = new StringBuilder();
            StringBuilder untranslatedLanguagesBuilder = new StringBuilder();
            foreach (KeyValuePair<TranslationItem, bool> translatedItem in _translateTasks)
            {
                if (translatedItem.Value == false)
                {
                    untranslatedLanguagesBuilder.Append(translatedItem.Key.LanguageName + ", ");
                }
                else
                {
                    translatedLanguagesBuilder.Append(translatedItem.Key.LanguageName + ", ");
                }
            }

            string untranslatedLanguages = untranslatedLanguagesBuilder.ToString();
            string translatedLanguages = translatedLanguagesBuilder.ToString();
            if (!string.IsNullOrEmpty(translatedLanguages))
            {
                translatedLanguages = translatedLanguages.Substring(0, translatedLanguages.Length - 2);
            }
            if (!string.IsNullOrEmpty(untranslatedLanguages))
            {
                untranslatedLanguages = untranslatedLanguages.Substring(0, untranslatedLanguages.Length - 2);
            }

            if (!string.IsNullOrEmpty(untranslatedLanguages))
            {
                task.TaskSubject = ApprovalBase.Translate(Owner, "/workflows/translate/notcompletesubject");
                task.TaskDescription = string.Format(CultureInfo.InvariantCulture,
                    "{0} {1} {2}",
                    PageName,
                    ApprovalBase.Translate(Owner, "/workflows/translate/notcompletedescription"),
                    untranslatedLanguages);
            }
            else
            {
                task.TaskSubject = ApprovalBase.Translate(Owner, "/workflows/translate/completesubject");
                task.TaskDescription = string.Format(System.Globalization.CultureInfo.InvariantCulture,
                    "{0} {1} {2}",
                    PageName,
                    ApprovalBase.Translate(Owner, "/workflows/translate/completedescription"),
                    translatedLanguages);
            }
        }

        /// <summary>
        /// Code condition that specifies if a page needs to be published before task is considered fulfilled
        /// </summary>
        private void IfNeedsPublish(object sender, ConditionalEventArgs e)
        {
            e.Result = this.PublishNeeded;
        }

        /// <summary>
        /// When a page is Published or Saved
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnPageEvent(object sender, WorkflowPageEventArgs e)
        {
            if (PageArgs.PageLink.CompareToIgnoreWorkID(e.PageLink))
            {
                if (e.EventType == WorkflowPageEvent.PagePublished ||
                    (e.EventType == WorkflowPageEvent.PageSaved && !PublishNeeded))
                {
                    PageData page = EPiServer.DataFactory.Instance.GetPage(e.PageLink);
                    foreach (KeyValuePair<TranslationItem, bool> translateItem in _translateTasks)
                    {
                        if (translateItem.Key.LanguageId == page.LanguageBranch)
                        {
                            if (IsAssignedTranslator(page.ChangedBy, translateItem.Key.Translator))
                            {
                                RemoveTask(translateItem.Key);
                                _translateTasks[translateItem.Key] = true;
                            }
                            return;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Code condition that indicates if an owner is specified
        /// </summary>
        private void IsOwnerSpecified(object sender, ConditionalEventArgs e)
        {
            e.Result = !string.IsNullOrEmpty(this.Owner);
        }


        /// <summary>
        /// Updates the created tasks DueDate. The reason we have to do this is becuase for a Task the reminder
        /// is sent when 3/4 of the time set in DueDate has elapsed. But since we want to have control over when 
        /// the reminder is sent we first created the task with duedate set so reminder get sent when we want it.
        /// Then (that is wath this activity does) we change the duedate to what we wanted.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void UpdateTaskDueDate(object sender, EventArgs e)
        {
            //Get all tasks for this instance
            IList<Task> tasks = Task.List(this.WorkflowInstanceId);
            foreach (Task task in tasks)
            {
                task.DueDate = _startTime.Add(_final);
                task.Save(false);
            }
        }


      
	}

}
