#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using EPiServer.Personalization;
using EPiServer.Core;
using EPiServer.PlugIn;
using EPiServer.Util.PlugIns;

using CMS6Workflows;
using EPiServer.WorkflowFoundation.UI;
using EPiServer.WorkflowFoundation;


namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// This control is loaded when a person clicks on an approval task.
    /// </summary>
    public partial class WorkflowApprovalEvent : System.Web.UI.UserControl, IWorkflowTaskControl
    {
        #region IWorkflowTaskControl Members
        /// <summary>
        /// Implementation of IWorkflowTaskControl.InvokeEvent that will be called when user clicks Send (if input is valid).
        /// The ApprovalService (must be registered e.g. in web.config)is received from runtime and event is invoked on 
        /// service to pass event to workflow instance. 
        /// </summary>
        /// <param name="workflowInstanceId">the workflow instance</param>
        /// <param name="taskId">the task id</param>
        /// <param name="eventQualifiedName">The Qualified name of the event activity that specified usercontrol</param>
        /// <returns>true if task should be deleted</returns>
        public bool InvokeEvent(Guid workflowInstanceId, int taskId, string eventQualifiedName)
        {
            //Get currentuser
            string currentUser = (System.Web.HttpContext.Current != null && EPiServer.Security.PrincipalInfo.Current != null) ?
                    EPiServer.Security.PrincipalInfo.Current.Name : string.Empty;

            Task task = Task.Load(taskId);

            //Invoke event on service
            ApprovalService service = WorkflowSystem.WorkflowManager.GetService<ApprovalService>();
            service.InvokeApprovalEvent(new ApprovalEventArgs(Approved.Checked, currentUser, task.AssignedTo, Message.Text, workflowInstanceId, VersionControl.GetSelectedVersion()));

            //If page version has changed, log it on instance
            VersionControl.LogIfVersionChanged(workflowInstanceId);

            //for sequential workflows delete task. For parallell delete task if it was a personal task, if it was assigned to a group don't delete it
            return IsParallellApproval(workflowInstanceId) ? !task.AssignedIsRole : true;
        }

        /// <summary>
        /// Will be called at startup of the control to sets the context for the task. Initialize controls.
        /// </summary>
        /// <param name="workflowInstanceId">The workflow instance id.</param>
        /// <param name="taskId">The task id.</param>
        /// <param name="eventQualifiedName">The Qualified name of the event activity that specified usercontrol</param>
        /// <param name="pageLink">The page that is currently loaded in beside action window</param>
        public void ContextData(Guid workflowInstanceId, int taskId, string eventQualifiedName, PageReference pageLink)
        {
            VersionControl.WorkflowPageLink = pageLink;
            History.WorkflowInstanceId = workflowInstanceId;
            History.AccessLevel = EPiServer.Security.AccessLevel.NoAccess;
            HistoryPanel.DataBind();
        }

        public void PageHasChanged(PageReference previousPageLink, PageReference newPageLink)
        {
            //If it is same page but different version we notify version control
            if (newPageLink.ID == VersionControl.WorkflowPageLink.ID && newPageLink.WorkID != VersionControl.WorkflowPageLink.WorkID)
            {
                VersionControl.VersionHasChanged(previousPageLink, newPageLink);
            }
        }

        /// <summary>
        /// Gets the text the invoke button should have. 
        /// If null or string.Empty is returned, default text "Send" (language dependent) is used
        /// </summary>
        /// <value>The invoke button text.</value>
        public string InvokeButtonText
        {
            get { return string.Empty; }
        }
        #endregion

        #region Accessors
        /// <summary>
        /// States if history panel should be displayed
        /// </summary>
        protected bool ShowHistory
        {
            get { return History.WorkflowInstanceId != Guid.Empty; }
        }
      
        /// <summary>
        /// The page name for the page that is approved
        /// </summary>
        protected string PageName
        {
            get
            {
                return !PageReference.IsNullOrEmpty(VersionControl.WorkflowPageLink)?
                      EPiServer.DataFactory.Instance.GetPage(VersionControl.WorkflowPageLink).PageName :
                      string.Empty;
            }
        }

        /// <summary>
        /// Determines whether specified workflow instance is of type ParallelApproval
        /// </summary>
        /// <param name="workflowInstanceId">The workflow instance id.</param>
        /// <returns>
        /// 	<c>true</c> if is of type ParallelApproval; otherwise, <c>false</c>.
        /// </returns>
        protected bool IsParallellApproval(Guid workflowInstanceId)
        {
            WorkflowDefinitionInstance instance = WorkflowSystem.InstanceHandler.GetInstance(workflowInstanceId, EPiServer.Security.AccessLevel.NoAccess);
            return instance.Type == typeof(ParallelApproval);
        }
        #endregion
    }

}