#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using EPiServer.WorkflowFoundation.UI;
using System.Collections.Generic;
using EPiServer.Web.PageExtensions;
using EPiServer.Core;
using EPiServer.Personalization;
using System.Collections.ObjectModel;
using System.Text;
using EPiServer.WorkflowFoundation;
using EPiServer.Security;

namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Control that is used both to set start parameters (implements IWorkflowStartParameterHandler) and to set approvers
    /// through task (implements IWorkflowTaskControl).
    /// </summary>
    /// <remarks>
    /// It is used both for Sequential and Parallel workflows but will display and behave a little differently depending on type.
    /// <para>
    /// It is rather complex due to the fact that it is used by two different workflows and both to specify start parameter as well 
    /// as task control. It is possible to split it to several (simpler) controls instead but then we get duplicated code.
    /// </para>
    /// </remarks>
    public partial class WorkflowApprovalStart : System.Web.UI.UserControl, IWorkflowStartParameterHandler, IWorkflowTaskControl
    {
        private List<ApproverItem> _approvers = new List<ApproverItem>();
        private bool _definitionMode = false;
        private bool _isParallelApproval;

        #region ViewState handling
        protected override void LoadViewState(object savedState)
        {
            object[] viewstates = savedState as object[];
            _approvers = (List<ApproverItem>)viewstates[0];
            _definitionMode = (bool)viewstates[1];
            _isParallelApproval = (bool)viewstates[2];
            base.LoadViewState(viewstates[3]);
        }
        protected override object SaveViewState()
        {
            object[] viewstates = new object[4] 
                { _approvers, _definitionMode, _isParallelApproval, base.SaveViewState() };
            return viewstates;
        }
        #endregion

        #region Accessors
        
        /// <summary>
        /// Will hold the current list of selected approvers
        /// </summary>
        protected List<ApproverItem> Approvers
        {
            get { return _approvers; }
            set { _approvers = value; }
        }

        /// <summary>
        /// Will hold the page for the approval workflow, if loaded in admin mode this will be PageReference.Empty
        /// </summary>
        protected PageReference CurrentPageLink
        {
            get { return VersionControl.WorkflowPageLink; }
            set { VersionControl.WorkflowPageLink = value; }
        }

        /// <summary>
        /// States whether the control is loaded in definition mode (that is admin mode) or in action window
        /// </summary>
        protected bool DefinitionMode
        {
            get { return _definitionMode; }
            set { _definitionMode = value; }
        }

        /// <summary>
        /// Indicates if the control is loaded for a parallel or a sequential workflow
        /// </summary>
        protected bool IsParallelApproval
        {
            get { return _isParallelApproval; }
            set { _isParallelApproval = value; }
        }

        /// <summary>
        /// The path to edit directory. Needed to load dialog for user/role selection
        /// </summary>
        protected string EditPath
        {
            get { return EPiServer.UriSupport.ResolveUrlFromUIBySettings("edit"); }
        }

        /// <summary>
        /// Indicates if History panel should be visible. It will be true when 
        /// control is loaded for a running workflow instance.
        /// </summary>
        protected bool ShowHistory
        {
            get { return History.WorkflowInstanceId != Guid.Empty; }
        }

        /// <summary>
        /// The name of the page the approval workflow is handling. 
        /// </summary>
        protected string PageName
        {
            get
            {
                return !PageReference.IsNullOrEmpty(CurrentPageLink) ?
                      EPiServer.DataFactory.Instance.GetPage(CurrentPageLink).PageName :
                      LanguageManager.Instance.Translate("/workflows/approval/startparameters/nopage");
            }
        }

        /// <summary>
        /// Indicates if it should be possible to set minimum number of approvers when control is loaded in 
        /// action window. False means a user cannot change the settings made in definition mode (admin mode)
        /// </summary>
        protected bool SelectNumbersVisible
        {
            get
            {
                if (!IsParallelApproval)
                {
                    return false;
                }
                else if (DefinitionMode)
                {
                    return true;
                }
                else
                {
                    return NumbersVisible.Checked;
                }
            }
        }

        /// <summary>
        /// Indicates if it should be possible to set approvers when control is loaded in action window.
        /// False means a user cannot change the settings made in definition mode (admin mode)
        /// </summary>
        protected bool SelectApproversVisible
        {
            get
            {
                if (DefinitionMode)
                {
                    return true;
                }
                else
                {
                    return ApproversVisible.Checked;
                }
            }
        }
        #endregion
        
        /// <summary>
        /// Since this control is dynamically loaded each postback, we databind every time
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (Request.Form["__EVENTTARGET"] == "UpdateUserList")
            {
                InsertUsers(Users.Value, Types.Value);
            }
            else if (Request.Form["__EVENTTARGET"] != null && Request.Form["__EVENTTARGET"].Contains("Grid"))
            {
                //In some occations eventhandling for checkbox inside grid will not be hooked up correctly. 
                //Instead of using this we do own event handling
                LookUpIfCheckBoxIsChecked(Request.Form["__EVENTTARGET"]);
            }

            CreateButtonColumns();

            HistoryPanel.DataBind();
            SelectApprovers.DataBind();
            SelectNumbers.DataBind();

            //Since we do own postback we want postback methods
            this.Page.ClientScript.GetPostBackEventReference(this, string.Empty);

            DataBindGrid();
        }
        #region Event handlers
        /// <summary>
        /// For parallel workflows we handle if a approver is required. That is workflow will 
        /// not complete even if minimum number of approvers is reached if there is some 
        /// required that has not approved.
        /// </summary>
        private void LookUpIfCheckBoxIsChecked(string controlId)
        {
            Control ctrl = this.Page.FindControl(controlId);
            if (ctrl != null && ctrl is CheckBox)
            {
                CheckBox checkBox = (CheckBox)ctrl;
                DataGridItem dataItem = checkBox.Parent.NamingContainer as DataGridItem;
                if (dataItem != null)
                {
                    Approvers[dataItem.ItemIndex].IsRequired = checkBox.Checked;
                }
            }
        }

        /// <summary>
        /// Noop implementation, used just to trigger postback. We handle the postback our self
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void IsRequiredChanged(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// For sequential workflows we handle the order of approvers
        /// </summary>
        protected void HandleButtons(Object sender, DataGridCommandEventArgs e)
        {
            switch (e.CommandName)
            {
                case "Up":
                    if (e.Item.ItemIndex != 0)
                    {
                        ApproverItem item = Approvers[e.Item.ItemIndex];
                        Approvers.RemoveAt(e.Item.ItemIndex);
                        Approvers.Insert(e.Item.ItemIndex - 1, item);
                    }

                    break;

                case "Down":
                    if (e.Item.ItemIndex != Approvers.Count - 1)
                    {
                        ApproverItem item = Approvers[e.Item.ItemIndex];
                        Approvers.RemoveAt(e.Item.ItemIndex);
                        Approvers.Insert(e.Item.ItemIndex + 1, item);
                    }
                    break;
            }
            DataBindGrid();
        }

        /// <summary>
        /// Hanldes delete user button click from approver list
        /// </summary>
        protected void RemoveUser(Object sender, DataGridCommandEventArgs e)
        {
            Approvers.RemoveAt(e.Item.ItemIndex);
            DataBindGrid();
        }

        /// <summary>
        /// Manual postback handling, we set in posted users/groups in users list
        /// </summary>
        /// <param name="usersString"></param>
        /// <param name="typesString"></param>
        private void InsertUsers(string usersString, string typesString)
        {
            string[] users = usersString.Split(';');
            string[] types = typesString.Split(';');
            for (int i = 0; i < users.Length; i++)
            {
                if (!string.IsNullOrEmpty(users[i]))
                {
                    Workflows.ApproverItem item = new ApproverItem(users[i], types[i] == "1", false);
                    Approvers.Add(item);
                }
            }
        }
        #endregion

        #region private helper methods
        /// <summary>
        /// Databinds approvers list
        /// </summary>
        private void DataBindGrid()
        {
            Grid.DataSource = Approvers;
            Grid.DataBind();
            UsersArea.Visible = Approvers.Count > 0;
        }

        /// <summary>
        /// For parallel approval we add buttons to let user decide approval order
        /// </summary>
        private void CreateButtonColumns()
        {
            if (!IsParallelApproval)
            {
                Grid.Columns.RemoveAt(2);
                Grid.Columns.AddAt(0, CreateLinkButtonColumn("Up", this.GetImageThemeUrl("Tools/Up.gif"), LanguageManager.Instance.Translate("/admin/categories/moveup")));
                Grid.Columns.AddAt(1, CreateLinkButtonColumn("Down", this.GetImageThemeUrl("Tools/Down.gif"), LanguageManager.Instance.Translate("/admin/categories/movedown")));
            }
            Grid.Columns.Add(CreateLinkButtonColumn("Delete", this.GetImageThemeUrl("Tools/Delete.gif"), LanguageManager.Instance.Translate("/button/delete")));
        }

        /// <summary>
        /// Creates a button column in approver list
        /// </summary>
        private ButtonColumn CreateLinkButtonColumn(string commandName, string imagePath, string tooltip)
        {
            ButtonColumn buttonColumn = new ButtonColumn();
            buttonColumn.ButtonType = ButtonColumnType.LinkButton;
            buttonColumn.Text = "<img src=\"" + imagePath + "\" border=\"0\" alt=\"" + tooltip + "\">";
            buttonColumn.HeaderStyle.CssClass = "epitableheading";
            buttonColumn.HeaderText = tooltip;
            buttonColumn.CommandName = commandName;
            buttonColumn.HeaderStyle.HorizontalAlign = HorizontalAlign.Center;
            buttonColumn.ItemStyle.HorizontalAlign = HorizontalAlign.Center;
            return buttonColumn;
        }

        /// <summary>
        /// Gets the image for the button form the used theme
        /// </summary>
        private string GetImageThemeUrl(string image)
        {
            return ThemeUtility.GetImageThemeUrl(this.Page, image);
        }

        #endregion

    
        #region IWorkflowStartParameterHandler Members
        /// <summary>
        /// Will be called at startup to set previous saved values. 
        ///  
        /// The values exposed by approval workflows is (name - type):
        /// Approvers - List&lt;ApproverItem&gt; (list of approvers) 
        /// TaskDescription - string (the description for the approvers)
        /// NeededApprovments - int (the minimum number of approvments for workflow,
        ///                     only used by ParallelApproval
        /// </summary>
        /// <param name="definitionId">The definition id of the selected definition</param>
        /// <param name="pageLink">The current page displayed, will be PageReferece.Empty in admin mode, that is when definitionMode is true</param>
        /// <param name="definitionMode">set to true if control is loaded to predefine params, that is in admin mode. 
        /// else if control is loaded when a user is about to create a workflow instance value is false</param>
        /// <param name="parameters">StartParameters</param>
        public void LoadStartParameters(
            Guid definitionId, 
            PageReference pageLink, 
            bool definitionMode, 
            System.Collections.Generic.IDictionary<string, object> parameters)
        {
            if (parameters.ContainsKey("Approvers") && parameters["Approvers"].GetType() == typeof(List<ApproverItem>))
            {
                Approvers = ((List<ApproverItem>)parameters["Approvers"]);
            }

            if (parameters.ContainsKey("TaskDescription") && parameters["TaskDescription"].GetType() == typeof(string))
            {
                Description.Text = (string)parameters["TaskDescription"];
            }

            if (parameters.ContainsKey("NeededApprovments") && parameters["NeededApprovments"].GetType() == typeof(int))
            {
                NumberOfApprovers.Text = ((int)parameters["NeededApprovments"]).ToString();
            }

            //We use StartParamters collection to save information about which fileds should be visible in action window
            //These values will never be sent down to workflow, they are only used to handle which fileds that should be
            //visible when control is loadad in action window.
            if (parameters.ContainsKey("ApproversVisible"))
            {
                ApproversVisible.Checked = (bool)parameters["ApproversVisible"];
            }

            if (parameters.ContainsKey("NumbersVisible"))
            {
                NumbersVisible.Checked = (bool)parameters["NumbersVisible"];
            }

            //Check if definition is parallel or sequential approval
            WorkflowDefinition definition = WorkflowSystem.DefinitionHandler.GetDefinition(definitionId);
            IsParallelApproval = definition.Type == typeof(ParallelApproval);

            DefinitionMode = definitionMode;
            CurrentPageLink = pageLink;
        }

        /// <summary>
        /// Will be called after user clicked Save/Start to get the values from the loaded control.
        /// </summary>
        /// <returns>StartParameters</returns>
        /// <remarks>Before SaveStartParameters are called Page.IsValid() is called so any custom validators will have chance to validate input.
        /// We dont have to set PageArgs, this will be set automatically by runtime</remarks>
        public System.Collections.Generic.IDictionary<string, object> SaveStartParameters()
        {
            Dictionary<string, object> startParameters = new Dictionary<string, object>();
            startParameters["Approvers"] = Approvers;
            startParameters["TaskDescription"] = Description.Text;
            if (!string.IsNullOrEmpty(NumberOfApprovers.Text))
            {
                startParameters["NeededApprovments"] = Int32.Parse(NumberOfApprovers.Text);
            }

            if (DefinitionMode)
            {
                //We use StartParamters collection to save information about which fileds should be visible in action window
                //These values will never be sent down to workflow, they are only used to handle which fileds that should be
                //visible when control is loadad in action window.
                startParameters["ApproversVisible"] = ApproversVisible.Checked;
                startParameters["NumbersVisible"] = NumbersVisible.Checked;
            }
            return startParameters;
        }

        #endregion



        #region IWorkflowTaskControl Members 
        /// <summary>
        /// Will be called at startup of the control to sets the context for the task. 
        /// This will be called when a approval workflow is not approved and task is 
        /// created for owner.
        /// </summary>
        /// <param name="workflowInstanceId">The workflow instance id.</param>
        /// <param name="taskId">The task id.</param>
        /// <param name="eventQualifiedName">The Qualified name of the event activity that specified usercontrol</param>
        /// <param name="pageLink">The page that is currently loaded in beside action window, that is the page that is being approved</param>
        /// <remarks>This will be called when control is loaded from task</remarks>
        public void ContextData(Guid workflowInstanceId, int taskId, string eventQualifiedName, PageReference pageLink)
        {
            CurrentPageLink = pageLink;

            WorkflowDefinitionInstance instance = WorkflowSystem.InstanceHandler.GetInstance(
                workflowInstanceId, 
                EPiServer.Security.AccessLevel.NoAccess);

            //Get previous values from workflow instance. They are set from workflow so we can initialize
            //control with same values at it had previously
            if (instance.UserData.ContainsKey("Approvers") && !IsPostBack)
            {
                Approvers = ((List<ApproverItem>)instance.UserData["Approvers"]);
            }

            if (instance.UserData.ContainsKey("NumbersOfApprovers") && !IsPostBack)
            {
                NumberOfApprovers.Text = ((int)instance.UserData["NumbersOfApprovers"]).ToString();
            }

            if (instance.UserData.ContainsKey("ApproversDescription") && !IsPostBack)
            {
                Description.Text = (string)instance.UserData["ApproversDescription"];
            }

            WorkflowDefinition definition = WorkflowSystem.DefinitionHandler.GetDefinition(instance.DefinitionId);
            //We use StartParamters collection to save information about which fields should be visible in action window
            //These values will never be sent down to workflow, they are only used to handle which fields that should be
            //visible when control is loadad in action window.
            if (definition.StartParameters.ContainsKey("ApproversVisible"))
            {
                ApproversVisible.Checked = (bool)definition.StartParameters["ApproversVisible"];
            }
            else
            {
                ApproversVisible.Checked = true;
            }

            if (definition.StartParameters.ContainsKey("NumbersVisible"))
            {
                NumbersVisible.Checked = (bool)definition.StartParameters["NumbersVisible"];
            }
            else
            {
                NumbersVisible.Checked = true;
            }

            //Check if this is parallel or sequential approval
            IsParallelApproval = instance.Type == typeof(ParallelApproval);
            
            History.WorkflowInstanceId = workflowInstanceId;
            History.AccessLevel = EPiServer.Security.AccessLevel.NoAccess;
            History.DataBind();
        }



        /// <summary>
        /// Will be called from framework if the page loaded in edit panel has changed since control was loaded
        /// </summary>
        /// <param name="previousPageLink">The previous page link.</param>
        /// <param name="newPageLink">The new page link.</param>
        public void PageHasChanged(PageReference previousPageLink, PageReference newPageLink)
        {
            //check if page is same but version has changed, if so notify version control
            if (newPageLink.ID == CurrentPageLink.ID && newPageLink.WorkID != CurrentPageLink.WorkID)
            {
                VersionControl.VersionHasChanged(previousPageLink, newPageLink);
            }
        }


        /// <summary>
        /// Method that is called when user clicks "Send" button. Sends data to SetApproversEvent activity.
        /// This will be called when a approval workflow is not approved and task is created for owner.
        /// </summary>
        /// <param name="workflowInstanceId">workflow instance</param>
        /// <param name="taskId">id of related task</param>
        /// <param name="eventQualifiedName">The Qualified name of the event activity that specified usercontrol</param>
        /// <returns>true if task should be deleted, else false</returns>
        /// <remarks>Before InvokeEvent are called Page.IsValid() is called so any custom validators will have chance to validate input</remarks>
        public bool InvokeEvent(Guid workflowInstanceId, int taskId, string eventQualifiedName)
        {
            //Get the ApprovalService from runtime (must be registerd e.g. in web.config) and invoke SetApproversEvent
            //on service to pass value down to workflow instance.
            ApprovalService service = WorkflowSystem.WorkflowManager.GetService<ApprovalService>();
            service.InvokeSetApproversEvent(
                new SetApproversEventArgs(
                    Description.Text, 
                    Approvers, 
                    string.IsNullOrEmpty(NumberOfApprovers.Text) ? 
                        Approvers.Count : Int32.Parse(NumberOfApprovers.Text),
                    VersionControl.GetSelectedVersion(), 
                    workflowInstanceId)
                    );
            
            //Add action to history of instance
            WorkflowDefinitionInstance instance = WorkflowSystem.InstanceHandler.GetInstance(workflowInstanceId, EPiServer.Security.AccessLevel.NoAccess);
            instance.AddHistoryItem(new HistoryItem("/workflows/approval/approvaltask/subject", Description.Text, PrincipalInfo.CurrentPrincipal.Identity.Name));
            WorkflowSystem.InstanceHandler.SaveInstance(instance, AccessLevel.NoAccess);
            
            
            //If page version has changed, log it on instance
            VersionControl.LogIfVersionChanged(workflowInstanceId);

            //Return true to delete task
            return true;
        }

        /// <summary>
        /// Gets the text the invoke button should have. 
        /// If null or string.Empty is returned, default text "Send" (language dependent) is used
        /// </summary>
        /// <value>The invoke button text.</value>
        public string InvokeButtonText
        {
            get { return string.Empty; }
        }


        #endregion

        #region validation
        /// <summary>
        /// Validates that a page is selected if control is loaded in action window
        /// </summary>
        protected void CustomValidation(object source, ServerValidateEventArgs args)
        {
            if (!DefinitionMode && PageReference.IsNullOrEmpty(CurrentPageLink))
            {
                args.IsValid = false;
                CustomValidator.ErrorMessage = LanguageManager.Instance.Translate("/workflows/approval/startparameters/notvalidpage");
            }
            else
            {
                args.IsValid = true;
            }
        }

        /// <summary>
        /// Validates that there are approvers selected if control is loaded in action window
        /// and that selected approvers has publish rights
        /// </summary>
        /// <remarks>In definition mode we can not check for publish right since at this moment we dont know which page to check for</remarks>
        protected void ApproversValidation(object source, ServerValidateEventArgs args)
        {
            args.IsValid = true;
        
            if (Approvers.Count == 0)
            {
                if (!DefinitionMode)
                {
                    args.IsValid = false;
                    ApproversValidator.ErrorMessage = LanguageManager.Instance.Translate("/workflows/approval/startparameters/noapprovers");
                }
            }
            else if(!DefinitionMode)
            {
                //We cant check for publish rights at design time since at that time we dont know which page.
                //instead the ApprovalService will perform the check at InstanceStarting event.
                ApprovalService approvalService = WorkflowSystem.WorkflowManager.GetService<ApprovalService>();
                StringBuilder usersWithoutPublishRights = new StringBuilder();
                foreach (ApproverItem approver in Approvers)
                {
                    if (!approvalService.HasReadAndPublishRights(approver.Approver, CurrentPageLink))
                    {
                        args.IsValid = false;
                        usersWithoutPublishRights.Append(approver.Approver + ", ");
                    }
                }
                if (!args.IsValid)
                {
                    ApproversValidator.ErrorMessage = string.Format(System.Globalization.CultureInfo.InvariantCulture,
                        LanguageManager.Instance.Translate("/workflows/approval/startparameters/notpublishrights"),
                        usersWithoutPublishRights.ToString(),
                        PageName);
                }
            }
           
        }

        /// <summary>
        /// Validates that if control is loaded in admin mode and user selects that it should not be possible 
        /// to set approvers in action window then there must be some predefined approvers set.
        /// </summary>
        protected void ApproversVisibleValidation(object source, ServerValidateEventArgs args)
        {
            if (DefinitionMode && Approvers.Count == 0 && !ApproversVisible.Checked)
            {
                args.IsValid = false;
                ApproversVisibleValidator.ErrorMessage = LanguageManager.Instance.Translate("/workflows/approval/startparameters/noapproversvisible");
            }
            else
            {
                args.IsValid = true;
            }
        }

        /// <summary>
        /// Validates that if minimum number of approvers is set that it is a positive number.
        /// If loaded in action window it is also validated that minimum number is not larger
        /// than total number of approvers (if there are no groups/roles given)
        /// </summary>
        protected void NumberValidation(object source, ServerValidateEventArgs args)
        {
            args.IsValid = true;
            if (!string.IsNullOrEmpty(NumberOfApprovers.Text))
            {
                int number;
                if (!Int32.TryParse(NumberOfApprovers.Text, out number))
                {
                    args.IsValid = false;
                    NumberOfApproversValidation.ErrorMessage =
                        LanguageManager.Instance.Translate("/workflows/approval/startparameters/notnumber");
                    return;
                }
                if (number <= 0)
                {
                    args.IsValid = false;
                    NumberOfApproversValidation.ErrorMessage =
                        LanguageManager.Instance.Translate("/workflows/approval/startparameters/positivenumber");
                    return;
                }
                else if (number > Approvers.Count && !ExistGroups)
                {
                    if (!DefinitionMode || !ApproversVisible.Checked)
                    {
                        args.IsValid = false;
                        NumberOfApproversValidation.ErrorMessage =
                            LanguageManager.Instance.Translate("/workflows/approval/startparameters/tolarge");
                        return;
                    }
                }
            }
        }

        /// <summary>
        /// Checks if there exists any groups in approvers list
        /// </summary>
        private bool ExistGroups
        {
            get
            {
                foreach (ApproverItem item in Approvers)
                {
                    if (item.IsRole)
                    {
                        return true;
                    }
                }
                return false;
            }
        }
        #endregion



    }
}