#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using EPiServer.WorkflowFoundation.UI;
using System.Collections.Generic;
using EPiServer.Web.PageExtensions;
using EPiServer.Core;
using EPiServer.Personalization;
using EPiServer.WorkflowFoundation.Activities;
using EPiServer.DataAbstraction;
using EPiServer.WorkflowFoundation;
using EPiServer;

namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Control that controls wether the currently loaded page version in edit panel differs
    /// from the page version for the workflow instance. If so validation fails and the user
    /// is asked to select which page version the workflow should use.
    /// </summary>
    public partial class WorkflowApprovalVersionControl : System.Web.UI.UserControl
    {
        /// <summary>
        /// Gets or sets the page version that is currently used by the workflow instance
        /// </summary>
        /// <value>The workflow page link.</value>
        public PageReference WorkflowPageLink
        {
            get
            {
                return WorkflowVersion.PageLink;
            }
            set
            {
                WorkflowVersion.PageLink = value;
            }
        }

        /// <summary>
        /// Gets or sets the current loaded page version in edit panel
        /// </summary>
        /// <value>The current loaded page link.</value>
        protected PageReference CurrentLoadedPageLink
        {
            get
            {
                return ViewState["CurrentLoadedPageLink"] != null ? (PageReference)ViewState["CurrentLoadedPageLink"] : PageReference.EmptyReference;
            }
            set
            {
                ViewState["CurrentLoadedPageLink"] = value;
            }
        }

        /// <summary>
        /// Will be called from framework when page version in edit panel has changed
        /// </summary>
        /// <param name="previousVersion">The previous version.</param>
        /// <param name="currentVersion">The current version.</param>
        public void VersionHasChanged(PageReference previousVersion, PageReference currentVersion)
        {
            CurrentLoadedPageLink = currentVersion;
        }

        /// <summary>
        /// Returns the version user has selected
        /// </summary>
        /// <returns>selected page version</returns>
        public PageReference GetSelectedVersion()
        {
            if (HasChanged)
            {
                return LoadedVersion.PageLink;
            }
            else
            {
                return WorkflowVersion.PageLink;
            }
        }

        /// <summary>
        /// If the version has changed this method logs the version change on workflow instance history log
        /// </summary>
        /// <param name="workflowInstanceId">The workflow instance id.</param>
        public void LogIfVersionChanged(Guid workflowInstanceId)
        {
            if (HasChanged)
            {
                string currentUser = (System.Web.HttpContext.Current != null && EPiServer.Security.PrincipalInfo.Current != null) ?
                    EPiServer.Security.PrincipalInfo.Current.Name : string.Empty;
                
                WorkflowDefinitionInstance instance = WorkflowSystem.InstanceHandler.GetInstance(workflowInstanceId, EPiServer.Security.AccessLevel.NoAccess);
                instance.AddHistoryItem(new HistoryItem(
                    LanguageManager.Instance.Translate("/workflows/approval/versioncontrol/changedsubject"),
                    string.Format(LanguageManager.Instance.Translate("/workflows/approval/versioncontrol/changeddescription"), WorkflowVersion.PageLink.ToString(), LoadedVersion.PageLink.ToString()),
                    currentUser));
                WorkflowSystem.InstanceHandler.SaveInstance(instance, EPiServer.Security.AccessLevel.NoAccess);

            }
        }



        /// <summary>
        /// Validates if the current loaded page version differs from the version used in the workflow instance. And if so displays the 
        /// versions to let the user select which version the workflow instance should process.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="args">The <see cref="System.Web.UI.WebControls.ServerValidateEventArgs"/> instance containing the event data.</param>
        /// <remarks>The first time a version is changed the version selection is shown to user. Next postback if loaded version has
        /// not changed again the version user selected will be the one used by workflow instance</remarks>
        public void ValidatePageVersion(object source, ServerValidateEventArgs args)
        {
            if (LoadedVersion.PageLink == PageReference.EmptyReference)
            {
                //We have not had a previous conflict, check if there is a different version loaded
                if (CurrentLoadedPageLink != PageReference.EmptyReference)
                {
                    //The loaded version differs from workflow instance version
                    RaiseValidationError(args);
                    return;
                }
            }
            else
            {
                //we have had a previous version conflict, check that loaded version has not changed again
                if (CurrentLoadedPageLink != LoadedVersion.PageLink)
                {
                    //User has browsed to yet another version again
                    RaiseValidationError(args);
                    return;
                }
            }
            VersionConflict.Visible = false;
        }

        /// <summary>
        /// Raises the validation error.
        /// </summary>
        /// <param name="args">The <see cref="System.Web.UI.WebControls.ServerValidateEventArgs"/> instance containing the event data.</param>
        private void RaiseValidationError(ServerValidateEventArgs args)
        {
            VersionConflict.Visible = true;
            LoadedVersion.PageLink = CurrentLoadedPageLink;
            args.IsValid = false;
            VersionValidator.ErrorMessage = LanguageManager.Instance.Translate("/workflows/approval/versioncontrol/differs");
            return;
        }

        /// <summary>
        /// Gets the javascript to navigate edit panel to the page version used by the workflow instance
        /// </summary>
        /// <value>The workflow version link.</value>
        protected string WorkflowVersionLink
        {
            get
            {
                return "window.top.navigateEvent(window.parent.parent.frames['EditTree'], '" + WorkflowVersion.PageLink + "');";
            }
        }

        /// <summary>
        /// Gets the name of the page version the workflow instance use.
        /// </summary>
        /// <value>The name of the workflow version.</value>
        protected string WorkflowVersionName
        {
            get
            {
                return DataFactory.Instance.GetPage(WorkflowVersion.PageLink).PageName;
            }
        }

        /// <summary>
        /// Gets the javascript to navigate edit panel to the page version that is/was loaded in edit panel
        /// </summary>
        /// <value>Link to the loaded version</value>
        protected string WorkflowLoadedLink
        {
            get
            {
                return "window.top.navigateEvent(window.parent.parent.frames['EditTree'], '" + LoadedVersion.PageLink + "');";
            }
        }

        /// <summary>
        /// Gets the name of the page version that was/is currently loaded in edit panel
        /// </summary>
        /// <value>The name of the workflow loaded.</value>
        protected string WorkflowLoadedName
        {
            get
            {
                return DataFactory.Instance.GetPage(LoadedVersion.PageLink).PageName;
            }
        }

        private bool HasChanged
        {
            get { return LoadedVersion.PageLink != PageReference.EmptyReference && LoadedVersion.Checked; }
        }

    }
}