#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using EPiServer.WorkflowFoundation.UI;
using System.Collections.Generic;
using EPiServer.Web.PageExtensions;
using EPiServer.Core;
using EPiServer.Personalization;
using EPiServer.WorkflowFoundation.Activities;
using System.Collections.ObjectModel;
using EPiServer.WorkflowFoundation;

namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Control that is used to configure start parameters for RequestForFeedback workflow
    /// </summary>
    public partial class WorkflowFeedbackStart : System.Web.UI.UserControl, IWorkflowStartParameterHandler
    {
        private List<UserItem> _users = new List<UserItem>();
        private bool _definitionMode = false;
        private PageReference _displayedPageVersion;

        #region ViewState handling
        protected override void LoadViewState(object savedState)
        {
            object[] viewstates = savedState as object[];
            _users = (List<UserItem>)viewstates[0];
            _definitionMode = (bool)viewstates[1];
            _displayedPageVersion = (PageReference)viewstates[2];
            base.LoadViewState(viewstates[3]);
        }
        protected override object SaveViewState()
        {
            object[] viewstates = new object[4] { _users, _definitionMode, _displayedPageVersion, base.SaveViewState() };
            return viewstates;
        }
        #endregion

        #region Accessors
        /// <summary>
        /// Currently selected users from which feedback is wanted
        /// </summary>
        protected List<UserItem> UserList
        {
            get { return _users; }
            set { _users = value; }
        }

        /// <summary>
        /// Indicates if control is loaded in definition mode (admin mode) or 
        /// in action window.
        /// </summary>
        protected bool DefinitionMode
        {
            get { return _definitionMode; }
            set { _definitionMode = value; }
        }

        /// <summary>
        /// The currently displayed page version in EditPanel.
        /// In admin mode this will be PageReference.EmptyReference
        /// </summary>
        protected PageReference DisplayedPageVersion
        {
            get { return _displayedPageVersion; }
            set { _displayedPageVersion = value; }
        }
        #endregion

        protected void Page_Load(object sender, EventArgs e)
        {
            //We handle postback from select user/group dialogs self
            if (Request.Form["__EVENTTARGET"] == "UpdateUserList")
            {
                InsertUsers(Users.Value, Types.Value);
            }
            else if (Request.Form["__EVENTTARGET"] == "UpdateOwner")
            {
                Owner.Text = Users.Value;
            }

            //Handle viewstate for page selector
            if (Request.Form[this.pageRoot.UniqueID + "$Hidden"] != null)
            {
                pageRoot.PageLink = string.IsNullOrEmpty(Request.Form[this.pageRoot.UniqueID + "$Hidden"]) ?
                    PageReference.EmptyReference :
                    new PageReference(Request.Form[this.pageRoot.UniqueID + "$Hidden"]);
            }

            CreateButtonColumns();

            //since we do manually postbacks from javascript we need postback method
            this.Page.ClientScript.GetPostBackEventReference(this, string.Empty);

            DataBindUserList();
        }
   
        protected void RemoveUser(Object sender, DataGridCommandEventArgs e)
        {
            UserList.RemoveAt(e.Item.ItemIndex);
            DataBindUserList();
        }

        private void DataBindUserList()
        {
            TranslatorsArea.Visible = UserList.Count > 0;
            Grid.DataSource = UserList;
            Grid.DataBind();
        }

        /// <summary>
        /// handles postback from User/Group selection dialog
        /// </summary>
        /// <param name="usersString"></param>
        /// <param name="typesString"></param>
        private void InsertUsers(string usersString, string typesString)
        {
            string[] users = usersString.Split(';');
            string[] types = typesString.Split(';');
            for (int i = 0; i < users.Length; i++)
            {
                if (!string.IsNullOrEmpty(users[i]))
                {
                    UserItem item = new UserItem(users[i], types[i] == "1");
                    UserList.Add(item);
                }
            }
        }

        private void CreateButtonColumns()
        {
            Grid.Columns.Add(CreateLinkButtonColumn("Delete", this.GetImageThemeUrl("Tools/Delete.gif"), LanguageManager.Instance.Translate("/button/delete")));
        }

        private ButtonColumn CreateLinkButtonColumn(string commandName, string imagePath, string tooltip)
        {
            ButtonColumn buttonColumn = new ButtonColumn();
            buttonColumn.ButtonType = ButtonColumnType.LinkButton;
            buttonColumn.Text = "<img src=\"" + imagePath + "\" border=\"0\" alt=\"" + tooltip + "\">";
            buttonColumn.HeaderStyle.CssClass = "epitableheading";
            buttonColumn.HeaderText = tooltip;
            buttonColumn.CommandName = commandName;
            buttonColumn.HeaderStyle.HorizontalAlign = HorizontalAlign.Center;
            buttonColumn.ItemStyle.HorizontalAlign = HorizontalAlign.Center;
            return buttonColumn;
        }

        private string GetImageThemeUrl(string image)
        {
            return ThemeUtility.GetImageThemeUrl(this.Page, image);
        }

        #region IWorkflowStartParameterHandler Members
        /// <summary>
        /// Will be called at startup to set previous saved values. 
        /// 
        /// The values exposed by RequestForFeedback workflow is (name - type):
        /// Users - List&lt;string&gt; (list of users feddback is wanted from) 
        /// RequestDescription - string (the description for the feedback request)
        /// Owner - string (the owner of the workflow, will recieve the feedback)
        /// TimeToLive - TimeSpan (the time that users will have to send feedback)
        /// PageArgs - WorkflowPageEventArgs (for manually started workflows this can be set, for automatically
        ///             started from a page related event this will be set by runtime)
        /// /// </summary>
        /// <param name="definitionId">The definition id of the selected definition</param>
        /// <param name="pageLink">The current page displayed, will be PageReferece.Empty in admin mode, that is when definitionMode is true</param>
        /// <param name="definitionMode">set to true if control is loaded to predefine params, that is in admin mode. 
        /// else if control is loaded when a user is about to create a workflow instance value is false</param>
        /// <param name="parameters">StartParameters</param>
        public void LoadStartParameters(Guid definitionId, PageReference pageLink, bool definitionMode, System.Collections.Generic.IDictionary<string, object> parameters)
        {
            if (parameters.ContainsKey("Users") && parameters["Users"].GetType() == typeof(List<UserItem>))
            {
                UserList = (List<UserItem>)parameters["Users"];
            }

            if (parameters.ContainsKey("RequestDescription") && parameters["RequestDescription"].GetType() == typeof(string))
            {
                Description.Text = (string)parameters["RequestDescription"];
            }

            if (parameters.ContainsKey("Owner") && parameters["Owner"].GetType() == typeof(string) && !string.IsNullOrEmpty((string)parameters["Owner"]))
            {
                Owner.Text = (string)parameters["Owner"];
            }
            else if (!definitionMode)
            {
                //If we are in action window and no Owner is predefined, set currentuser as owner
                string currentUser = (System.Web.HttpContext.Current != null && EPiServer.Security.PrincipalInfo.Current != null) ?
                    EPiServer.Security.PrincipalInfo.Current.Name : string.Empty;
                Owner.Text = currentUser;
            }

            if (parameters.ContainsKey("TimeToLive") && parameters["TimeToLive"].GetType() == typeof(TimeSpan))
            {
                TimeToLive.Text = ConvertedTimespan(((TimeSpan)parameters["TimeToLive"]).TotalHours).ToString();
            }

            if (parameters.ContainsKey("PageArgs") && parameters["PageArgs"].GetType() == typeof(WorkflowPageEventArgs))
            {
                pageRoot.PageLink = ((WorkflowPageEventArgs)parameters["PageArgs"]).PageLink;
            }

            //if loaded from edit tab, set pagelink if not already set
            if (PageReference.IsNullOrEmpty(pageRoot.PageLink) && Page.Request["currentPage"] != null)
            {
                pageRoot.PageLink = pageLink;
            }

            DefinitionMode = definitionMode;
            DisplayedPageVersion = pageLink;
        }

        /// <summary>
        /// Will be called after user clicked Save/Start to get the values from the loaded control.
        /// </summary>
        /// <returns>StartParameters</returns>
        /// <remarks>Before SaveStartParameters are called Page.IsValid() is called so any custom validators will have chance to validate input.
        /// </remarks>
        public System.Collections.Generic.IDictionary<string, object> SaveStartParameters()
        {
            Dictionary<string, object> startParameters = new Dictionary<string, object>();
            startParameters["Users"] = UserList;

            startParameters["RequestDescription"] = Description.Text;
            startParameters.Add("TimeToLive", new TimeSpan(Int32.Parse(TimeToLive.Text), 0, 0));
            startParameters["Owner"] = Owner.Text;

            //Get page version, if it is page related
            if (!PageReference.IsNullOrEmpty(pageRoot.PageLink) && !DefinitionMode)
            {
                //special handling to get right version of page, since page picker control only takes published versions.
                if (pageRoot.PageLink.ID == DisplayedPageVersion.ID)
                {
                    pageRoot.PageLink = DisplayedPageVersion;
                }
            }
            startParameters["PageArgs"] = new WorkflowPageEventArgs(WorkflowPageEvent.None, pageRoot.PageLink, PageReference.EmptyReference,
                   string.Empty, Guid.Empty);

            return startParameters;
        }

        #endregion

        /// <summary>
        /// Validates that owner, description and Users are selected if control is 
        /// loaded in action window
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        protected void CustomValidation(object source, ServerValidateEventArgs args)
        {
            if (!DefinitionMode)
            {
                if (string.IsNullOrEmpty(Owner.Text))
                {
                    args.IsValid = false;
                    CustomValidator.ErrorMessage = LanguageManager.Instance.Translate("/workflows/feedbackrequest/noowner");
                }
                else if (string.IsNullOrEmpty(Description.Text))
                {
                    args.IsValid = false;
                    CustomValidator.ErrorMessage = LanguageManager.Instance.Translate("/workflows/feedbackrequest/nodescription");
                }
                else if (UserList.Count == 0)
                {
                    args.IsValid = false;
                    CustomValidator.ErrorMessage = LanguageManager.Instance.Translate("/workflows/feedbackrequest/nousers");
                } 
            }
            else
            {
                args.IsValid = true;
            }
        }

        /// <summary>
        /// Convert double to int
        /// </summary>
        private int ConvertedTimespan(double value)
        {
            return (int)Math.Ceiling(value);
        }

    }
}