#region Copyright  1997-2007 EPiServer AB. All Rights Reserved.
/*
This code may only be used according to the EPiServer License Agreement.
The use of this code outside the EPiServer environment, in whole or in
parts, is forbidden without prior written permission from EPiServer AB.

EPiServer is a registered trademark of EPiServer AB. For more information 
see http://www.episerver.com/license or request a copy of the EPiServer 
License Agreement by sending an email to info@episerver.com
*/
#endregion
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using EPiServer.Personalization;
using EPiServer.Core;
using EPiServer.PlugIn;
using EPiServer.Util.PlugIns;

using EPiServer.WorkflowFoundation;
using EPiServer.WorkflowFoundation.UI;
using System.Collections.Generic;
using EPiServer.Web.PageExtensions;
using EPiServer.DataAbstraction;
using System.Collections.ObjectModel;


namespace CMS6Workflows.Workflows
{
    /// <summary>
    /// Control that is used to configure start parameters for ReadyForTranslation workflow
    /// </summary>
    public partial class WorkflowTranslationStart : System.Web.UI.UserControl, IWorkflowStartParameterHandler
	{
        //Holds the list of currently explicitly selected translators
        private List<TranslationItem> _translators = new List<TranslationItem>();
        private PageReference _pageLink = PageReference.EmptyReference;

        #region ViewState handling
        protected override void LoadViewState(object savedState)
        {
            object[] viewstates = savedState as object[];
            _translators = (List<TranslationItem>)viewstates[0];
            _pageLink = (PageReference)viewstates[1];
            base.LoadViewState(viewstates[2]);
        }
        protected override object SaveViewState()
        {
            object[] viewstates = new object[3] { _translators, _pageLink, base.SaveViewState() };
            return viewstates;
        }
        #endregion

        #region Accessors
        /// <summary>
        /// Currently configured translators
        /// </summary>
        protected List<TranslationItem> Translators
        {
            get { return _translators; }
            set { _translators = value; }
        }
        /// <summary>
        /// The name of the page the translation workflow is handling. 
        /// </summary>
        protected string PageName
        {
            get
            {
                return !PageReference.IsNullOrEmpty(CurrentPageLink) ?
                      EPiServer.DataFactory.Instance.GetPage(CurrentPageLink).PageName :
                      LanguageManager.Instance.Translate("/workflows/approval/startparameters/nopage");
            }
        }
        /// <summary>
        /// Will hold the page for the translation workflow, if loaded in admin mode this will be PageReference.Empty
        /// </summary>
        protected PageReference CurrentPageLink
        {
            get { return _pageLink; }
            set { _pageLink = value; }
        }
        #endregion

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            //Handle manually triggered postbacks from Select user/group dialogs
            if (Request.Form["__EVENTTARGET"] == "UpdateTranslatorList")
            {
                InsertTranslator(PostedTranslator.Value, PostedLanguage.Value);
            }
            else if (Request.Form["__EVENTTARGET"] == "UpdateOwner")
            {
                Owner.Text = PostedTranslator.Value;
            }
            else if (Request.Form["__EVENTTARGET"] == "UpdateFallback")
            {
                Fallback.Text = PostedTranslator.Value;
            }

            //since we do manually postbacks from javascript we need postback method
            this.Page.ClientScript.GetPostBackEventReference(this, string.Empty);

            CreateDeleteButton();
            DataBindTranslators();
            SetNotSelectedLanguages();
        }

        /// <summary>
        /// Creates a delete button in translators grid
        /// </summary>
        private void CreateDeleteButton()
        {
            ButtonColumn buttonColumn = new ButtonColumn();
            buttonColumn.ButtonType = ButtonColumnType.LinkButton;
            buttonColumn.Text = "<img src=\"" + ThemeUtility.GetImageThemeUrl(this.Page, "Tools/Delete.gif") + "\" border=\"0\" alt=\"" + LanguageManager.Instance.Translate("/button/delete") + "\">";
            buttonColumn.HeaderStyle.CssClass = "epitableheading";
            buttonColumn.HeaderText = LanguageManager.Instance.Translate("/button/delete");
            buttonColumn.CommandName = "Delete";
            buttonColumn.HeaderStyle.HorizontalAlign = HorizontalAlign.Center;
            buttonColumn.ItemStyle.HorizontalAlign = HorizontalAlign.Center;
            Grid.Columns.Add(buttonColumn);
        }

        /// <summary>
        /// Databinds translators list
        /// </summary>
        private void DataBindTranslators()
        {
            if (Translators.Count == 0)
            {
                GridPanel.Visible = false;
            }
            else
            {
                GridPanel.Visible = true;
                Grid.DataSource = Translators;
                Grid.DataBind();
            }
        }

        /// <summary>
        /// Handles callback from javascript to insert a user in translators list
        /// </summary>
        /// <param name="username"></param>
        /// <param name="language"></param>
        private void InsertTranslator(string username, string language)
        {
            if (!string.IsNullOrEmpty(username) && !string.IsNullOrEmpty(language))
            {
                Translators.Add(new TranslationItem(username, LanguageBranch.Load(language).Name, language));
            }
            SetNotSelectedLanguages();
            DataBindTranslators();       
        }

      
        /// <summary>
        /// Fills language dropdown with those languages that are enabled and not already configured
        /// </summary>
        private void SetNotSelectedLanguages()
        {
            Languages.Items.Clear();
            LanguageBranchCollection languages = LanguageBranch.ListAll();
            foreach (LanguageBranch language in languages)
            {
                //Only enabled languages
                if (language.Enabled)
                {
                    //check that this language is not already configured
                    bool isConfigured = false;
                    foreach (TranslationItem item in Translators)
                    {
                        if (item.LanguageId == language.LanguageID)
                        {
                            isConfigured = true;
                            break;
                        }
                    }
                    if (!isConfigured)
                    {
                        Languages.Items.Add(new ListItem(language.Name, language.LanguageID));
                    }
                }
            }
        }

        /// <summary>
        /// Hanldes delete button clicked in translator list
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void RemoveTranslator(object sender, DataGridCommandEventArgs e)
        {
            Translators.RemoveAt(e.Item.ItemIndex);
            SetNotSelectedLanguages();
            DataBindTranslators();
        }

        #region IWorkflowStartParameterHandler Members
        /// <summary>
        /// Will be called at startup to set previous saved values.
        /// 
        /// The values exposed by ReadyForTranslation workflow is (name - type):
        /// InitialDelay - TimeSpan (time before translation requests are sent out)
        /// Reminder -TimeSpan (time before reminders are sent to translators)
        /// Final - TimeSpan (time before translation tasks are cleared and result is sent to owner)
        /// MainTranslator - string (Translator for not explicitly given languages)
        /// Owner - string (User who get notification of translation result)
        /// PublishNeeded - bool (indicates wheter the page has to be published in language before task is fulfilled)
        /// Translators - List&lt;TranslationItem&gt; (list of explicitly given translators) 
        /// </summary>
        /// <param name="definitionId">The definition id of the selected definition</param>
        /// <param name="pageLink">The current page displayed, will be PageReferece.Empty in admin mode, that is when definitionMode is true</param>
        /// <param name="definitionMode">set to true if control is loaded to predefine params, that is in admin mode. 
        /// else if control is loaded when a user is about to create a workflow instance value is false</param>
        /// <param name="parameters">StartParameters</param>
        public void LoadStartParameters(Guid definitionId, PageReference pageLink, bool definitionMode, System.Collections.Generic.IDictionary<string, object> parameters)
        {
            if (parameters.ContainsKey("InitialDelay") && parameters["InitialDelay"].GetType() == typeof(TimeSpan))
            {
                InitialDelay.Text = ConvertedTimespan(((TimeSpan)parameters["InitialDelay"]).TotalMinutes).ToString();
            }

            if (parameters.ContainsKey("Reminder") && parameters["Reminder"].GetType() == typeof(TimeSpan))
            {
                Reminder.Text = ConvertedTimespan(((TimeSpan)parameters["Reminder"]).TotalHours).ToString();
            }

            if (parameters.ContainsKey("Final") && parameters["Final"].GetType() == typeof(TimeSpan))
            {
                Terminate.Text = ConvertedTimespan(((TimeSpan)parameters["Final"]).TotalHours).ToString();
            }

            if (parameters.ContainsKey("MainTranslator") && parameters["MainTranslator"].GetType() == typeof(string))
            {
                Fallback.Text = (string)parameters["MainTranslator"];
            }

            if (parameters.ContainsKey("Owner") && parameters["Owner"].GetType() == typeof(string))
            {
                Owner.Text = (string)parameters["Owner"];
            }

            if (parameters.ContainsKey("PublishNeeded") && parameters["PublishNeeded"].GetType() == typeof(bool))
            {
                NeedsToBePublished.Checked = (bool)parameters["PublishNeeded"];
            }

            if (parameters.ContainsKey("Translators") && parameters["Translators"].GetType() == typeof(List<TranslationItem>))
            {
                Translators = (List<TranslationItem>)parameters["Translators"];
            }

            CurrentPageLink = pageLink;
        }

        /// <summary>
        /// Saves the start parameters. Called at "Save" in admin mode or "Start" in action window.
        /// </summary>
        /// <returns></returns>
        public System.Collections.Generic.IDictionary<string, object> SaveStartParameters()
        {
            Dictionary<string, object> startParameters = new Dictionary<string,object>();
            startParameters.Add("InitialDelay", new TimeSpan(0, Int32.Parse(InitialDelay.Text), 0));
            startParameters.Add("Reminder", new TimeSpan(Int32.Parse(Reminder.Text), 0, 0));
            startParameters.Add("Final", new TimeSpan(Int32.Parse(Terminate.Text), 0, 0));
            startParameters.Add("Translators", Translators);
            startParameters.Add("MainTranslator", Fallback.Text);
            startParameters.Add("Owner", Owner.Text);
            startParameters.Add("PublishNeeded", NeedsToBePublished.Checked);
            return startParameters;
        }

        /// <summary>
        /// Converts a double to int
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        private int ConvertedTimespan(double value)
        {
            return (int)Math.Ceiling(value);
        }

        #endregion
    }
}